/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiFrame;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.border.BevelBorder;

import org.apache.log4j.Logger;
import org.gvsig.gui.beans.controls.IControl;

import com.iver.andami.messages.Messages;
import com.iver.andami.plugins.config.generate.Label;


/**
 */
public class NewStatusBar extends JPanel {
	private static Logger logger = Logger.getLogger(NewStatusBar.class.getName());
	private static final int INFO = 0;
	private static final int WARNING = 1;
	private static final int ERROR = 2;
	private JLabel lblIcon = null;
	private JLabel lblTexto = null;
	private boolean contenidoTemporal;
	private String textoAnterior;
	private int estadoAnterior;
	private ImageIcon infoIcon;
	private ImageIcon warningIcon;
	private ImageIcon errorIcon;
	private int estado;
	private JProgressBar progressBar = null;
	private HashMap idLabel = new HashMap();
	private int[] widthlabels = null;
	private HashMap controls = new HashMap();

	/**
	 * This is the default constructor
	 */
	public NewStatusBar() {
		super();
		initialize();
		infoIcon = new ImageIcon(NewStatusBar.class.getClassLoader()
												   .getResource("images/info.gif"));
		warningIcon = new ImageIcon(NewStatusBar.class.getClassLoader()
													  .getResource("images/warning.gif"));
		errorIcon = new ImageIcon(NewStatusBar.class.getClassLoader()
													.getResource("images/error.gif"));
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		this.setPreferredSize(new java.awt.Dimension(183,20));
		this.setSize(new java.awt.Dimension(183,20));
		lblIcon = new JLabel();
		lblTexto = new JLabel();

		FlowLayout flowLayout2 = new FlowLayout();
		this.setLayout(flowLayout2);
		flowLayout2.setHgap(1);
		flowLayout2.setVgap(2);
		flowLayout2.setAlignment(java.awt.FlowLayout.LEFT);
		lblIcon.setText("");
		lblTexto.setText(Messages.getString("StatusBar.Aplicacion_iniciada"));
		this.add(lblIcon, null);
		this.add(getProgressBar(), null);
		this.add(lblTexto, null);
	}

	/**
	 * Obtiene el texto de la barra de estado
	 *
	 * @return texto
	 */
	public String getStatusText() {
		return lblTexto.getText();
	}

	/**
	 * Restaura el contenido permanente de la barra de estado
	 */
	public void restaurarTexto() {
		contenidoTemporal = false;

		if (estadoAnterior == -1) {
			return;
		}

		switch (estadoAnterior) {
			case INFO:
				setInfoText(textoAnterior);

				break;

			case WARNING:
				setWarningText(textoAnterior);

				break;

			case ERROR:
				setErrorText(textoAnterior);

				break;
		}

		estadoAnterior = -1;
		textoAnterior = null;
	}

	/**
	 * Establece el texto de la barra de estado de forma temporal. Se almacena
	 * el texto que haba en la barra de estado.
	 *
	 * @param texto texto
	 */
	public void setInfoTextTemporal(String texto) {
		contenidoTemporal = true;

		estadoAnterior = this.estado;
		this.estado = INFO;
		lblIcon.setIcon(infoIcon);

		textoAnterior = getStatusText();
		lblTexto.setText(texto);
	}

	/**
	 * Establece el texto de la barra de estado de forma temporal. Se almacena
	 * el texto que haba en la barra de estado.
	 *
	 * @param texto texto
	 */
	public void setWarningTextTemporal(String texto) {
		contenidoTemporal = true;

		estadoAnterior = this.estado;
		this.estado = WARNING;
		lblIcon.setIcon(warningIcon);

		textoAnterior = getStatusText();
		lblTexto.setText(texto);
	}

	/**
	 * Establece el texto de la barra de estado de forma temporal. Se almacena
	 * el texto que haba en la barra de estado.
	 *
	 * @param texto texto
	 */
	public void setErrorTextTemporal(String texto) {
		contenidoTemporal = true;

		estadoAnterior = this.estado;
		this.estado = ERROR;
		lblIcon.setIcon(errorIcon);

		textoAnterior = getStatusText();
		lblTexto.setText(texto);
	}

	/**
	 * Establece el texto de la barar de manera permanente. Si se est
	 * mostrando texto de forma temporal no se actualiza la interfaz
	 *
	 * @param texto Texto
	 */
	public void setInfoText(String texto) {
		if (contenidoTemporal) {
			textoAnterior = texto;
			estadoAnterior = INFO;
		} else {
			lblTexto.setText(texto);
			lblIcon.setIcon(infoIcon);
			estado = INFO;
		}
	}

	/**
	 * Establece el texto de la barar de manera permanente. Si se est
	 * mostrando texto de forma temporal no se actualiza la interfaz
	 *
	 * @param texto Texto
	 */
	public void setWarningText(String texto) {
		if (contenidoTemporal) {
			textoAnterior = texto;
			estadoAnterior = WARNING;
		} else {
			lblTexto.setText(texto);
			lblIcon.setIcon(warningIcon);
			estado = WARNING;
		}
	}

	/**
	 * Establece el texto de la barar de manera permanente. Si se est
	 * mostrando texto de forma temporal no se actualiza la interfaz
	 *
	 * @param texto Texto
	 */
	public void setErrorText(String texto) {
		if (contenidoTemporal) {
			textoAnterior = texto;
			estadoAnterior = ERROR;
		} else {
			lblTexto.setText(texto);
			lblIcon.setIcon(errorIcon);
			estado = ERROR;
		}
	}

	/**
	 * Establece el porcentaje en la barra de progreso. Si es 100 se oculta la
	 * barra
	 *
	 * @param p DOCUMENT ME!
	 */
	public void setProgress(int p) {
		if (p < 100) {
			getProgressBar().setValue(p);
			getProgressBar().setVisible(true);
		} else {
			getProgressBar().setVisible(false);
		}

		getProgressBar().repaint();
	}

	/*public void setControls(Label[] labels, ArrayList controlArray) {
		removeAllLabels();
		if (labels!=null)
			setLabelSet(labels);
		if (controlArray!=null)
			setControls(controlArray);
		this.repaint();
	}*/

	public void setLabelSet(Label[] labels) {
		removeAllLabels();
		idLabel.clear();

		for (int i = 0; i < labels.length; i++) {
			JLabel lbl = new JLabel();
			lbl.setPreferredSize(new Dimension(labels[i].getSize(),
					this.getHeight() - 2));
			lbl.setSize(new Dimension(labels[i].getSize(), this.getHeight() - 2));
			lbl.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
			this.add(lbl);

			/*            if (i != labels.length - 1){
			   this.add(new JSeparator(JSeparator.VERTICAL));
			   }
			 */
			idLabel.put(labels[i].getId(), lbl);
		}

		JLabel[] configlabels = (JLabel[]) idLabel.values().toArray(new JLabel[0]);
		widthlabels = new int[configlabels.length];

		for (int i = 0; i < labels.length; i++) {
			widthlabels[i] = configlabels[i].getWidth();
		}

		this.repaint();
	}

	/*public void setControls(ArrayList controlList) {

		for (int i = 0; i < controlList.size(); i++) {
			Component control = (Component) controlList.get(i);
			control.setVisible(true);
			control.setEnabled(true);
			this.add(control);
		}
	}*/

	/**
	 * Los Labels que estan sin texto no se aaden a la barra y su espacio es
	 * utilizado por otros labels. Si se quiere hacer en un futuro la
	 * asignacin de espacio de los elementos que se aadan a la barra se
	 * puede hacer aqu.
	 */
	public void ajustar() {
		if (widthlabels == null) {
			return;
		}

		int ws = this.getWidth();

		JLabel[] labels = (JLabel[]) idLabel.values().toArray(new JLabel[0]);

		/*        double total = 1;
		   for (int i = 0; i < widthlabels.length; i++) {
		           if (labels[i].getText().compareTo("") != 0) {
		                   total += widthlabels[i];
		           }
		   }
		   double p = (ws - lblTexto.getWidth() - 20) / total;
		 */
		for (int i = 0; i < labels.length; i++) {
			//if (labels[i] instanceof JLabel){
			JLabel label = (JLabel) labels[i];

			if (label.getText().compareTo("") != 0) {
				label.setVisible(true);
				label.setPreferredSize(new Dimension((int) (widthlabels[i]),
						this.getHeight() - 2));
			} else {
				label.setVisible(false);

				//label.setPreferredSize(new Dimension(0,this.getHeight()));
			}

			//}
		}
	}

	/**
	 * Removes all the labels from the status bar.
	 */
	private void removeAllLabels() {
		Component[] controlArray = this.getComponents();

		for (int i = 0; i < controlArray.length; i++) {
			if ((controlArray[i] != lblIcon) && (controlArray[i] != lblTexto && controlArray[i] instanceof JLabel)) {
				remove(controlArray[i]);
			}
		}
	}


	/**
	 * Removes all the controls (including labels) from the status bar.
	 */
	private void removeAllControls() {
		Component[] controlArray = this.getComponents();

		for (int i = 0; i < controlArray.length; i++) {
			if ((controlArray[i] != lblIcon) && (controlArray[i] != lblTexto)) {
				remove(controlArray[i]);
			}
		}
	}

	/**
	 * Establece el texto de una de las etiquetas
	 *
	 * @param id Identificador de las etiquetas
	 * @param msg Mensaje que se quiere poner en la etiqueta
	 */
	public void setMessage(String id, String msg) {
		JLabel lbl = (JLabel) idLabel.get(id);

		if (lbl == null) {
//			logger.debug("no label called " + id);
		} else {
			lbl.setText(msg);
		}

		ajustar();
	}

	/**
	 * Sets the control identified by 'id' with the provided value.
	 *
	 * @param id
	 * @param value
	 */
	public void setControlValue(String id, String value) {
		IControl control = (IControl) controls.get(id);
		if (control!=null) {
			control.setValue(value);
		}
		else {
			logger.debug("NewStatusBar -- no control called " + id);
		}
	}

	/**
	 * This method initializes progressBar
	 *
	 * @return javax.swing.JProgressBar
	 */
	private JProgressBar getProgressBar() {
		if (progressBar == null) {
			progressBar = new JProgressBar();
			progressBar.setPreferredSize(new java.awt.Dimension(100, 14));
			progressBar.setVisible(false);
			progressBar.setMinimum(0);
			progressBar.setMaximum(100);
			progressBar.setValue(50);
		}

		return progressBar;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param d
	 */
	public void setFixedLabelWidth(double d) {
		lblTexto.setPreferredSize(new Dimension((int) d, lblTexto.getHeight()));
	}

	/**
	 * Adds a control to the status bar
	 */
	public void addControl(String id, Component control) {
		this.add(control);
		if (!controls.containsKey(control.getName()))
				controls.put(control.getName(), control);
		else
			logger.debug("NewStatusBar.addControl -- control 'id' already exists"+ id);
	}

	/**
	 * Gets a control from the status bar
	 */
	public Component getControl(String id, Component control) {
		return (Component) controls.get(id);
	}
} //  @jve:decl-index=0:visual-constraint="10,10"
