package com.iver.andami.ui.theme;

import org.kxml2.io.KXmlParser;

import org.xmlpull.v1.XmlPullParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import java.util.ArrayList;

import javax.swing.ImageIcon;


/**
 * Personalization of the program according to the file xml with the necessary information.
 *
 * @author Vicente Caballero Navarro
 */
public class Theme {
    public static final String CENTERED = "CENTERED";
    public static final String EXPAND = "EXPAND";
    public static final String MOSAIC = "MOSAIC";

	private static String encoding = "UTF-8";
	private static final String ANDAMI_PROPERTIES = "AndamiProperties";
    private static final String APPLICATION_IMAGES = "ApplicationImages";
    private static final String SPLASH_IMAGES = "SplashImages";
    private static final String SPLASH = "Splash";
    private static final String PATH = "path";
    private static final String TIMER = "timer";
    private static final String ICON = "Icon";
    private static final String APPLICATION_NAME = "ApplicationName";
    private static final String VALUE = "value";
	private static final String BACKGROUND_IMAGE = "BackgroundImage";
	private static final String WALLPAPER_TYPE = "WallpaperType";
    private ArrayList images = new ArrayList();
    private String icon;
    private ArrayList timers = new ArrayList();
    private String name = null;
	private String backgroundimage;
	private String wallpaperType=CENTERED;

    public static void main(String[] args) {
        Theme theme = new Theme();
        theme.readTheme(new File(
                "c:/workspace/_fwAndami/theme/andami-theme.xml"));
    }

    /**
     * Read the file xml with the personalization.
     *
     * @param file xml
     */
    public void readTheme(File file) {
        try {
            FileReader fr;

            try {
                fr = new FileReader(file);

                BufferedReader br = new BufferedReader(fr);
                char[] buffer = new char[100];
                br.read(buffer);

                StringBuffer st = new StringBuffer(new String(buffer));
                String searchText = "encoding=\"";
                int index = st.indexOf(searchText);

                if (index > -1) {
                    st.delete(0, index + searchText.length());
                    encoding = st.substring(0, st.indexOf("\""));
                }
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            fr = new FileReader(file);

            KXmlParser parser = new KXmlParser();
            parser.setInput(new FileInputStream(file), encoding);

            int tag = parser.nextTag();

            if (parser.getEventType() != KXmlParser.END_DOCUMENT) {
                parser.require(KXmlParser.START_TAG, null, ANDAMI_PROPERTIES);

                while (tag != KXmlParser.END_DOCUMENT) {
                    //parser.next();
                    switch (tag) {
                    case KXmlParser.START_TAG:

                        if (parser.getName().compareTo(ANDAMI_PROPERTIES) == 0) {
                            parser.nextTag();
                            if (parser.getName().compareTo(APPLICATION_IMAGES) == 0) {
                                int splashtag = parser.nextTag();
                            if (parser.getName().compareTo(SPLASH_IMAGES) == 0) {
									splashtag = parser.nextTag();
									boolean endSplash = false;

									// parser.require(KXmlParser.START_TAG,
									// null, SPLASH);
									while ((splashtag != KXmlParser.END_DOCUMENT)
											&& !endSplash) {
										if (splashtag == KXmlParser.END_TAG) {
											splashtag = parser.nextTag();

											continue;
										}

										if (parser.getName().compareTo(SPLASH) == 0) {
											images
													.add(parser
															.getAttributeValue(
																	"", PATH));
											timers.add(parser
													.getAttributeValue("",
															TIMER));
											System.out.println(images.size()
													+ " " + timers.size());
											splashtag = parser.nextTag();
										} else {
											endSplash = true;
										}
									}
								}
								int tagOptions = KXmlParser.START_TAG;
								while ((tagOptions != KXmlParser.END_TAG)) {
									if (parser.getName().compareTo(
											BACKGROUND_IMAGE) == 0) {
										backgroundimage = parser
												.getAttributeValue("", PATH);
										tag = parser.next();
									} else if (parser.getName().compareTo(ICON) == 0) {
										icon = parser.getAttributeValue("",
												PATH);
										tag = parser.next();
									} else if (parser.getName().compareTo(
											WALLPAPER_TYPE) == 0) {
										wallpaperType = parser.getAttributeValue(
												"", VALUE);
										tag = parser.next();
									}
									tagOptions = parser.nextTag();

								}
							}
                        }
                        if (parser.getName().compareTo(APPLICATION_NAME) == 0) {
							name = parser.getAttributeValue("",
									VALUE);
							tag = parser.next();
						}
                        break;

                    case KXmlParser.END_TAG:
                        break;

                    case KXmlParser.TEXT:

                        // System.out.println("[TEXT]["+kxmlParser.getText()+"]");
                        break;
                    }

                    tag = parser.next();
                }
            }

            parser.require(XmlPullParser.END_DOCUMENT, null, null);

            // }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("error");
        }
    }

    /**
	 * Returns image to the splashwindow.
	 *
	 * @return ImageIcon[]
	 */
    public ImageIcon[] getSplashImages() {
        ImageIcon[] imgs = new ImageIcon[images.size()];

        for (int i = 0; i < images.size(); i++) {
            imgs[i] = new ImageIcon((String) images.get(i));
        }

        return imgs;
    }

    public String getTypeDesktop() {
    	return wallpaperType;
    }
    /**
     * Return the icon.
     *
     * @return ImageIcon
     */
    public ImageIcon getIcon() {
        if (icon == null) {
            return null;
        }

        try {
            return new ImageIcon(icon);
        } catch (Exception e) {
            return null;
        }
    }
    /**
     * Return the backgroundimage.
     *
     * @return ImageIcon
     */
    public ImageIcon getBackgroundImage() {
        if (backgroundimage == null) {
            return null;
        }

        try {
            return new ImageIcon(backgroundimage);
        } catch (Exception e) {
            return null;
        }
    }
    /**
     * Return the time of the splash images.
     *
     * @return long[]
     */
    public long[] getTimers() {
        long[] tms = new long[timers.size()];

        for (int i = 0; i < tms.length; i++) {
            tms[i] = Long.parseLong((String) timers.get(i));
        }

        return tms;
    }

    /**
     * Return the name of program.
     *
     * @return String
     */
    public String getName() {
        return name;
    }
}
