package es.gva.cit.catalogClient;

import java.io.File;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;

import com.iver.utiles.swing.jcomboServer.ServerData;

import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTreeNumberOfRecordsAnswer;
import es.gva.cit.catalogClient.querys.Query;

import junit.framework.TestCase;
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: CatalogTest.java,v 1.1.2.1 2006/11/15 04:10:43 jjdelcerro Exp $
 * $Log: CatalogTest.java,v $
 * Revision 1.1.2.1  2006/11/15 04:10:43  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.1  2006/11/14 08:50:43  jorpiell
 * Aadido un test para las bsquedas de catlogo
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class CatalogTest extends TestCase {
	private String server1 = "http://www.gvsig.org:2100";
	private String protocol1 = ServerData.SERVER_SUBTYPE_CATALOG_Z3950;
	private String database1 = "";	
		
	/**
	 * Busca los parques (1 resultado)
	 */
	public void testServer1(){
		Query query = new Query();
		query.setTitle("parques");
		//Esto lo que hace es buscar en el campo nombre y descripcin(maximiza los 
		//resultados de la bsqueda
		query.setMinimized(true);		
		Collection results = doSearch(server1,protocol1,database1,query,1);
		assertEquals(results.size(),2);
	}
	
	/**
	 * Busca las comunicaciones (6 resultados)
	 * @return
	 */
	public void testServer2(){
		Query query = new Query();
		query.setTitle("comunicaciones");
		query.setMinimized(true);		
		Collection results = doSearch(server1,protocol1,database1,query,1);
		assertEquals(results.size(),7);
	}
	
	/**
	 * Busca albalat (1 Resultado)
	 * @return
	 */
	public void testServer3(){
		Query query = new Query();
		query.setTitle("albalat");
		query.setMinimized(true);
		Collection results = doSearch(server1, protocol1, database1, query,1);
		assertEquals(results.size(), 2);
	}
	
	/**
	 * Busca albalat (1 Resultado)
	 * @return
	 */
	public void testServer4(){
		Query query = new Query();
		query.setTitle("XXX");
		//query.setMinimized(true);
		Collection results = doSearch(server1, protocol1, database1, query,1);
		assertEquals(results.size(), 1);
	}
	
	/**
	 * Busca todos los resultados (recupera los nodos de 10 en 10 tres veces, es decir,
	 * busca los 30 primeros resultados) y escribe por pantalla el resultado 31.
	 */
	public void testServer5(){
		Query query = new Query();
		query.setTitle(null);
		query.setMinimized(true);		
		Collection results = doSearch(server1,protocol1,database1,query,1);
		assertEquals(results.size(),11);
		results = doSearch(server1,protocol1,database1,query,11);
		assertEquals(results.size(),11);
		results = doSearch(server1,protocol1,database1,query,21);
		assertEquals(results.size(),11);
		((XMLNode)results.toArray()[1]).printNode();
	}	

	/**
	 * Escribe los resultados de la bsqueda por pantalla. Los resultados se
	 * recuperan de 10 en 10.
	 * @param collection
	 */
	private void printSearchResults(Collection collection){
		System.out.println("***** NUEVA BUSQUEDA *******");
		XMLNode node = (XMLNode)collection.toArray()[0];
		System.out.println("RESULTADOS OBTENIDOS: " + XMLTreeNumberOfRecordsAnswer.getNumberOfRecords(node));
		System.out.println("PRIMER RERSULTADO: " + XMLTreeNumberOfRecordsAnswer.getFirstRecord(node));
		System.out.println("ULTIMO RESULTADO: " + XMLTreeNumberOfRecordsAnswer.getLastRecord(node));		
	}
	
	/**
	 * Hace una bsqueda y devuelve una coleccin de resultados. En el elemento
	 * primero de la coleccin se devuelve un rbol en XML que contiene los resultados 
	 * de la bsqueda, que son el nmero total de coincidencias, la posicin del 
	 * primer elemento recuperado y la posicin del ltimo elemento recuperado.
	 * @param url
	 * URL del servidor
	 * @param protocol
	 * Protocolo del servidor
	 * @param database
	 * Base de datos a conectarse
	 * @param query
	 * Pregunta a realizar
	 * @param firstRecord
	 * Primer registro a recuperar
	 * @return
	 * Coleccion de resultados obtenidos + un nodo extra que contiene la informacin de la
	 * bsqueda
	 */
	private Collection doSearch(String url, String protocol, String database,Query query,int firstRecord){
		CatalogClient client = new CatalogClient(url,protocol,database);
		try {
			Collection results = client.getRecords(new URL(url),query, firstRecord);
			printSearchResults(results);
			return results;
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
		
	}

}
