package es.gva.cit.gazetteer.wfsg.drivers;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;

import com.iver.utiles.swing.jcomboServer.ServerData;

import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.querys.Coordinates;
import es.gva.cit.gazetteer.ISearchByName;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import es.gva.cit.gazetteer.wfsg.parsers.WfsgFeatureParser;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSGSearchByName.java,v 1.4 2006/12/12 12:21:09 jorpiell Exp $
 * $Log: WFSGSearchByName.java,v $
 * Revision 1.4  2006/12/12 12:21:09  jorpiell
 * Se ha sacado la operacin capabilities del constructor
 *
 * Revision 1.3  2006/12/12 11:45:55  maquerol
 * Opciones de visualizacion
 *
 * Revision 1.2  2006/12/12 11:32:20  jorpiell
 * Se ha hecho una comprobacin de la proyeccin
 *
 * Revision 1.1  2006/12/12 10:20:51  jorpiell
 * Se ha creado una interfaz que deben usar las tools del gazetteer
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSGSearchByName extends WFSGazetteerServiceDriver implements ISearchByName{
	private Query currentQuery = null;
	private String serverURL = "http://www.gvsig.org:9080/deegree-wfsg/wfsg";
	private boolean isCapabilities = false;
		
	public WFSGSearchByName() {        
		super();		
	} 
	
	private String getFeatureName() {
		return "nomenclator_cv";
	}

	/**
	 * It searches and returns an array of features.
	 * 
	 * 
	 * @return 
	 * @param placeName String with the place name to find.
	 */
	public Feature[] search(String placeName) {        
		if (!isCapabilities){
			try {
				setParameters(getCapabilities(new URL(getServerURL())));			
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			isCapabilities = true;
		}
		currentQuery = new Query(placeName,getFeatureName());
		currentQuery.setFieldAttribute("geographicIdentifier");
		currentQuery.getOptions().getSearch().setWithAccents(true);
		currentQuery.getOptions().getAspect().setGoTo(true);
		currentQuery.getOptions().getAspect().setPaintCurrent(true);
		setProjection("EPSG:23030");		
		try {
			Collection nodes = getFeature(new URL(getServerURL()),currentQuery);
			Feature[] features = new WfsgFeatureParser().parse((XMLNode)nodes.toArray()[0],getThesaurus(),null);
			return features;
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		
	} 
	
	private ThesaurusName getThesaurus(){
		ThesaurusName thesaurus = new ThesaurusName();
		thesaurus.setSrs("EPSG:23030");
		thesaurus.setName("nomenclator_cv");
		thesaurus.setTitle("Comunidad Valenciana");	
		return thesaurus;
	}
	
	/**
	 * 
	 * 
	 * 
	 * @return Returns the serverURL.
	 */
	public String getServerURL() {        
		return serverURL;
	} 
	
	/**
	 * 
	 * 
	 * 
	 * @param serverURL The serverURL to set.
	 */
	public void setServerURL(String serverURL) {        
		this.serverURL = serverURL;
	}
	
	/**
	 * @return Returns the currentQuery.
	 */
	public Query getCurrentQuery() {
		return currentQuery;
	}

	public String getServerType() {
		return ServerData.SERVER_SUBTYPE_GAZETTEER_WFSG;
	} 
	
	
}
