/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project.documents.view.toolListeners;

import java.awt.Cursor;
import java.awt.geom.Point2D;
import java.text.NumberFormat;

import org.cresques.cts.IProjection;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiFrame.MainFrame;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PointListener;


/**
 * DOCUMENT ME!
 *
 * @author Vicente Caballero Navarro
 */
public class StatusBarListener implements PointListener {
	private MapControl mapControl = null;
	private NumberFormat nf = null;

	/**
	 * Crea un nuevo StatusBarListener.
	 *
	 * @param mc DOCUMENT ME!
	 */
	public StatusBarListener(MapControl mc) {
		mapControl = mc;
		nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
		return false;
	}

	/**
	 * 050211, jmorell: Mtodo modificado para mejorar la manera de mostrar las
	 * coordenadas geodsicas en la barra de estado. Muestra Lat y Lon y aumenta
	 * el nmero de decimales para cuando trabajemos en coordenadas geodsicas.
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PointListener#point(com.iver.cit.gvsig.fmap.tools.Events.PointEvent)
	 */
	public void point(PointEvent event) throws BehaviorException {
		String[] axisText = new String[2];
		axisText[0] = "X = ";
		axisText[1] = "Y = ";
		Point2D p = mapControl.getMapContext().getViewPort().toMapPoint(event.getPoint());
		setFractionDigits(p);
		axisText = setCoorDisplayText(axisText);
		MainFrame mF = PluginServices.getMainFrame();

		if (mF != null)
		{
            mF.getStatusBar().setMessage("units",
            		PluginServices.getText(this, FConstant.NAMES[mapControl.getMapContext().getViewPort().getDistanceUnits()]));
            // FJP: No se debe llamar a setControlValue desde aqu, porque
            // cambia la escala, y con ella el viewPort (adems, de
            // la vista que no es).
            // mF.getStatusBar().setControlValue("scale",String.valueOf(mapControl.getMapContext().getScaleView()));
            // Fin
			mF.getStatusBar().setMessage("projection", mapControl.getViewPort().getProjection().getAbrev());

			mF.getStatusBar().setMessage("x",
					axisText[0] + String.valueOf(nf.format((p.getX()/MapContext.CHANGEM[mapControl.getViewPort().getDistanceUnits()])*MapContext.CHANGEM[mapControl.getViewPort().getMapUnits()])));
			mF.getStatusBar().setMessage("y",
					axisText[1] + String.valueOf(nf.format((p.getY()/MapContext.CHANGEM[mapControl.getViewPort().getDistanceUnits()])*MapContext.CHANGEM[mapControl.getViewPort().getMapUnits()])));
		}
	}

	/**
	 * Aumenta el nmero de decimales visibles cuando trabajamos con coordeandas
	 * geodsicas. En concreto se aaden 8 decimales, con lo que logramos una
	 * definicin submtrica.
	 * 050211, jmorell.
	 * @param p
	 */
	private void setFractionDigits(Point2D p) {
		IProjection iProj = mapControl.getMapContext().getProjection();
		if (iProj.getAbrev().equals("EPSG:4326") || iProj.getAbrev().equals("EPSG:4230")) {
			nf.setMaximumFractionDigits(8);
		} else {
			nf.setMaximumFractionDigits(2);
		}
	}

	/**
	 * Cambia X e Y por Lat y Lon enb la barra de estado cuando trabajamos con
	 * coordenadas geodsicas.
	 * 050211, jmorell.
	 * @param p
	 */
	private String[] setCoorDisplayText(String[] axisText) {
		IProjection iProj = mapControl.getMapContext().getProjection();
		if (iProj.getAbrev().equals("EPSG:4326") || iProj.getAbrev().equals("EPSG:4230")) {
			axisText[0] = "Lon = ";
			axisText[1] = "Lat = ";
		} else {
			axisText[0] = "X = ";
			axisText[1] = "Y = ";
		}
		return axisText;
	}

	public void pointDoubleClick(PointEvent event) throws BehaviorException {
		// TODO Auto-generated method stub

	}

}
