/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;

import javax.print.attribute.PrintRequestAttributeSet;

import org.cresques.filter.RasterFilterStack;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.geo.ViewPortData;
import org.cresques.io.GdalFile;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.data.Grid;
import org.cresques.io.exceptions.NotSupportedExtensionException;
import org.cresques.px.Extent;
import org.cresques.px.PxRaster;
import org.exolab.castor.xml.ValidationException;
import org.gvsig.remoteClient.wcs.WCSStatus;
import org.gvsig.remoteClient.wms.ICancellable;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.ConnectionErrorExceptionType;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.UnknownResponseFormatExceptionType;
import com.iver.cit.gvsig.fmap.UnsuportedProtocolVersionExceptionType;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.WCSDriverExceptionType;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.UnsupportedVersionException;
import com.iver.cit.gvsig.fmap.drivers.wcs.FMapWCSDriver;
import com.iver.cit.gvsig.fmap.drivers.wcs.FMapWCSDriverFactory;
import com.iver.cit.gvsig.fmap.drivers.wcs.WCSException;
import com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint;
import com.iver.cit.gvsig.fmap.layers.layerOperations.StringXMLItem;
import com.iver.cit.gvsig.fmap.layers.layerOperations.XMLItem;
import com.iver.utiles.DoubleUtilities;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.swing.threads.Cancellable;


/**
 * Class for the WCS layer.
 *
 * Capa para el WCS.
 *
 * @author jaume - jaume.dominguez@iver.es
 */
public class FLyrWCS extends FLyrDefault implements RasterOperations, InfoByPoint{
	private FMapWCSDriver wcs = null;

	private URL 						host;
	private String						coverageName;
	private Rectangle2D					fullExtent;
	private String						format;
	private String						srs;
	private String						time;
	private String						parameter;
	private Point2D						maxRes;
	private Hashtable 					onlineResources = new Hashtable();

	private WCSStatus					wcsStatus = new WCSStatus();

	private StatusRasterInterface		status = null;
	private int 						posX = 0, posY = 0;
	private double 						posXWC = 0, posYWC = 0;
	private int 						r = 0, g = 0, b = 0;
	private RasterFilterStackManager	stackManager = null;
	private PxRaster 					raster = null;
	private GeoRasterFile 				rasterFile = null;
	private RasterFilterStack 			filterStack;
	private boolean 					firstLoad = false;
	private int							rband = 0, gband = 1, bband = 2;
	private VisualStatus				visualStatus = new VisualStatus();

	private boolean 					mustTileDraw = false;
	private int 						maxTileDrawWidth  = 1023;
	private int							maxTileDrawHeight = 1023;
	private int 						maxTilePrintWidth  = 250;
	private int							maxTilePrintHeight = 250;
	
	private Grid						grid = null;

	private class MyCancellable implements ICancellable
	{

		private Cancellable original;
		public MyCancellable(Cancellable cancelOriginal)
		{
			this.original = cancelOriginal;
		}
		public boolean isCanceled() {
			return original.isCanceled();
		}

	}
	public FLyrWCS(){
		super();
	}

	public FLyrWCS(Map args) throws DriverIOException{
		FMapWCSDriver drv = null;
		String host = (String)args.get("HOST");
		String sCoverage = (String) args.get((String) "COVERAGE");

		try {
			this.setHost(new URL(host));
		} catch (MalformedURLException e) {
			//e.printStackTrace();
			throw new DriverIOException("Malformed host URL, '" + host + "' (" + e.toString() + ").");
		}
		try {
			drv = this.getDriver();
		} catch (Exception e) {
			// e.printStackTrace();
			throw new DriverIOException("Can't get driver to host '" + host + "' (" + e.toString() + ").");
		}

		try{
			if (!drv.connect(false, null)){
				throw new DriverIOException("Can't connect to host '" + host + "'.");
			}
		}catch(Exception e){
			throw new DriverIOException("Can't connect to host '" + host + "'.");
		}

		WCSLayer wcsNode = drv.getLayer(sCoverage);

		if (wcsNode == null){
			throw new DriverIOException("The server '" + host + "' doesn't has the coverage '" + sCoverage + "'.");
		}

		try{
			this.setFullExtent(drv.getFullExtent(sCoverage,
					(String) args.get((String) "CRS")));
			this.setFormat((String) args.get((String) "FORMAT"));
			this.setParameter("BANDS=" + (String) args.get((String) "BANDS"));
			this.setSRS((String) args.get((String) "CRS"));
			this.setName(sCoverage);
			this.setCoverageName(sCoverage);
		}catch (Exception e){
			throw new DriverIOException("The server '" + host + "' is not able to load the coverage '" + sCoverage + "'.");
		}

	}

	/**
	 * Clase que contiene los datos de visualizacin de WCS. Tiene datos que representan al
	 * raster en la vista. Este raster puede estar compuesto por tiles por lo que valores
	 * como el ancho total o el mnimo o mximo deben ser calculados a partir de todos los
	 * tiles visualizados.
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
	private class VisualStatus {
		/**
		 * Ancho y alto de la imagen o del conjunto de tiles si los tiene. Coincide con
		 * el ancho y alto del viewPort
		 */
		private	int							width = 0, height = 0;
		private double						minX = 0D, minY = 0D, maxX = 0D, maxY = 0D;
		private int 						bandCount = 0;
		private int							dataType = DataBuffer.TYPE_UNDEFINED;

		/**
		 * Ancho y alto total del raster que ser la suma de todos los tiles.
		 */
		private	int							rasterWidth = 0, rasterHeight = 0;
		private	double						rasterMinX = Double.MAX_VALUE, rasterMinY = Double.MAX_VALUE;
		private	double						rasterMaxX = 0, rasterMaxY = 0;
		/**
		 * Lista de nombre de fichero que componen toda la visualizacin.
		 */
		private String[]					fileNames = null;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setRGB(int, int, int)
	 */
	public void setRGB(int r, int g, int b) {
		this.r = r;
		this.g = g;
		this.b = b;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getFilterStack()
	 */
	public RasterFilterStack getFilterStack() {
		if(raster!=null)
			return raster.filterStack;
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setFilterStack(org.cresques.io.raster.RasterFilterStack)
	 */
	public void setFilterStack(RasterFilterStack stack) {
		this.filterStack = stack;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getSource()
	 */
	public RasterAdapter getSource() {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setSource(com.iver.cit.gvsig.fmap.layers.RasterAdapter)
	 */
	public void setSource(RasterAdapter ra) {
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setPos(int, int)
	 */
	public void setPos(int x, int y) {
		this.posX = x;
		this.posY = y;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setPosWC(double, double)
	 */
	public void setPosWC(double x, double y) {
		this.posXWC = x;
		this.posYWC = y;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getPixel(double, double)
	 */
	public int[] getPixel(double wcx, double wcy) {
		if(getPxRaster() != null)
			return getPxRaster().getPixel(wcx, wcy);
        return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setBand(int, int)
	 */
	public void setBand(int flag, int nBand) {
		switch(flag){
		case GeoRasterFile.RED_BAND:setBandR(nBand);break;
		case GeoRasterFile.GREEN_BAND:setBandG(nBand);break;
		case GeoRasterFile.BLUE_BAND:setBandB(nBand);break;
		}
	}

	/**
	 * Sets the R-band.
	 *
	 * Asigna la banda R.
	 * @param r
	 */
	public void setBandR(int r){
		this.rband = r;
	}

	/**
	 * Sets the G-band.
	 *
	 * Asigna la banda G
	 * @param g
	 */
	public void setBandG(int g){
		this.gband = g;
	}

	/**
	 * Sets the B-band.
	 *
	 * Asigna la banda B
	 * @param b
	 */
	public void setBandB(int b){
		this.bband = b;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getAttributes()
	 */
	public ArrayList getAttributes() {
		if (rasterFile != null){
			ArrayList attr = new ArrayList();
			String dataType = "Byte";
			if (rasterFile.getDataType() == DataBuffer.TYPE_BYTE) dataType = "Byte";
			else if (visualStatus.dataType == DataBuffer.TYPE_SHORT)
				dataType = "Short";
			else if (visualStatus.dataType == DataBuffer.TYPE_USHORT)
				dataType = "Unsigned Short";
			else if (visualStatus.dataType == DataBuffer.TYPE_INT)
				dataType = "Integer";
			else if (visualStatus.dataType == DataBuffer.TYPE_FLOAT)
				dataType = "Float";
			else if (visualStatus.dataType == DataBuffer.TYPE_DOUBLE)
				dataType = "Double";
			else
				dataType = "Unknown";

			Object [][] a = {
				{"Filename",rasterFile.getName().substring(rasterFile.getName().lastIndexOf("/")+1, rasterFile.getName().length())},
				{"Filesize",new Long(0)},
				{"Width",new Integer((int)this.getWidth())},
				{"Height", new Integer((int)this.getHeight())},
				{"Bands", new Integer(visualStatus.bandCount)},
				{"BandDataType", dataType}
			};
			for (int i=0; i<a.length; i++)
				attr.add(a[i]);

			return attr;
		}
		return  null;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMaxX()
	 */
	public double getMaxX() {
		return visualStatus.maxX;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMaxY()
	 */
	public double getMaxY() {
		return visualStatus.maxY;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMinX()
	 */
	public double getMinX() {
		return visualStatus.minX;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMinY()
	 */
	public double getMinY() {
		return visualStatus.minY;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getWidth()
	 */
	public double getWidth() {
		return visualStatus.width;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getHeight()
	 */
	public double getHeight() {
		return visualStatus.height;
	}

	/**
	 * @deprecated
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint#getInfo
	 */
	public String queryByPoint(Point p) throws DriverException {
		String data = "<file:"+getName().replaceAll("[^a-zA-Z0-9]","")+">\n";
		ArrayList attr = this.getAttributes();
		data += "  <raster\n";
		data += "    File=\""+getName()+"\"\n";
		for (int i=0; i<attr.size(); i++) {
			Object [] a = (Object []) attr.get(i);

			data += "    "+a[0].toString()+"=";
			if (a[1].toString() instanceof String)
				data += "\""+a[1].toString()+"\"\n";
			else
				data += a[1].toString()+"\n";
		}
		data += "    Point=\""+posX+" , "+posY+"\"\n";
		data += "    Point_WC=\""+posXWC+" , "+posYWC+"\"\n";
		data += "    RGB=\""+r+", "+g+", "+b+"\"\n";
		data += "  />\n";

		data += "</file:"+getName().replaceAll("[^a-zA-Z0-9]","")+">\n";
		System.out.println(data);
		return data;
	}
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint#getInfo
	 */
	public XMLItem[] getInfo(Point point, double tolerance, Cancellable cancel ) throws DriverException {
		String data = "<file:"+getName().replaceAll("[^a-zA-Z0-9]","")+">\n";

		Point2D pReal = getMapContext().getViewPort().toMapPoint(point);
		Point2D px = null;
		if(	pReal.getX() > this.getMinX() &&
			pReal.getX() < this.getMaxX() &&
			pReal.getY() > this.getMinY() &&
			pReal.getY() < this.getMaxY()){

			px = new Point2D.Double();
			double w = (visualStatus.rasterMaxX - visualStatus.rasterMinX);
			double h = (visualStatus.rasterMaxY - visualStatus.rasterMinY);
			px.setLocation(((pReal.getX() - visualStatus.rasterMinX) * visualStatus.rasterWidth) / w,
							((visualStatus.rasterMaxY - pReal.getY()) * visualStatus.rasterHeight) / h);
		}

		int[] rgb = this.getPixel(pReal.getX(), pReal.getY());

		StringXMLItem[] item = new StringXMLItem[1];

		data += "  <raster\n";
		data += "    View_Point=\""+point.getX()+" , "+point.getY()+"\"\n";
		data += "    World_Point=\""+DoubleUtilities.format(pReal.getX(), 3)+" , "+ DoubleUtilities.format(pReal.getY(), 3)+"\"\n";
		if(	px == null || px.getX() > visualStatus.rasterWidth || px.getX() < 0 ||
				px.getY() > visualStatus.rasterHeight || px.getY() < 0)
				data += "    Pixel_Point=\"Out\"\n";
			else{
				data += "    Pixel_Point=\""+(int)px.getX()+" , "+(int)px.getY()+"\"\n";		
				data += "    Band_Value=\"";
				for(int file = 0; file < visualStatus.fileNames.length; file++ ){
					GdalFile rf;
					try {
						rf = new GdalFile(getMapContext().getViewPort().getProjection(), visualStatus.fileNames[file]);
					} catch (NotSupportedExtensionException e) {
						throw new DriverException("Open File Error.");
					}
					Extent ex = rf.getExtent();
					
					if(pReal.getX() >= ex.minX() && pReal.getX() <= ex.maxX() && pReal.getY() >= ex.minY() && pReal.getY() <= ex.maxY()){
						for(int i = 0; i < visualStatus.bandCount; i++){
							Object pxData = rf.getData((int)px.getX(), (int)px.getY(), i);
							if(pxData != null){
								if(visualStatus.dataType >= 0 && visualStatus.dataType <= 3)
									data += ((Integer)pxData).intValue()+"  ";
														
								if(visualStatus.dataType >= 4)
									data += ((Float)pxData).floatValue()+"  ";
									
								if(visualStatus.dataType >= 5)
									data += ((Double)pxData).doubleValue()+"  ";
							}
						}
					}
					rf.close();
				}
				data += "\"\n";
			}
			
			if(rgb != null)
				data += "    RGB=\""+rgb[1]+"  "+rgb[2]+"  "+rgb[3]+"\"\n";
			
			data += "  />\n";
			data += "</file:"+getName().replaceAll("[^a-zA-Z0-9]","")+">\n";

			item[0] = new StringXMLItem(data, this);
			return item;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getFullExtent()
	 */
	public Rectangle2D getFullExtent() {
		return fullExtent;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable, double)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale) throws DriverException {

		if (isWithinScale(scale)){
			Point2D p = viewPort.getOffset();
			// p will be (0, 0) when drawing a view or other when painting onto
			// the Layout.
			visualStatus.width =  viewPort.getImageWidth();
			visualStatus.height =  viewPort.getImageHeight();
			visualStatus.minX = viewPort.getAdjustedExtent().getMinX();
			visualStatus.minY = viewPort.getAdjustedExtent().getMinY();
			visualStatus.maxX = viewPort.getAdjustedExtent().getMaxX();
			visualStatus.maxY = viewPort.getAdjustedExtent().getMaxY();
			visualStatus.rasterWidth = 0;
			visualStatus.rasterHeight = 0;
			visualStatus.rasterMinX = Double.MAX_VALUE;
			visualStatus.rasterMinY = Double.MAX_VALUE;
			visualStatus.rasterMaxX = 0;
			visualStatus.rasterMaxY = 0;
			visualStatus.fileNames = new String[1];

			if(mustTileDraw){
				if(viewPort.getImageWidth()<= maxTileDrawWidth && viewPort.getImageHeight()<=maxTileDrawHeight)
					drawTile(g, viewPort, cancel, 0);
				else{
					Rectangle r = new Rectangle((int) p.getX(), (int) p.getY(), viewPort.getImageWidth(), viewPort.getImageHeight());
					Tiling tiles = new Tiling(maxTileDrawWidth, maxTileDrawHeight, r);
					tiles.setAffineTransform((AffineTransform) viewPort.getAffineTransform().clone());
					visualStatus.fileNames = new String[tiles.getNumTiles()];
					for (int tileNr=0; tileNr < tiles.getNumTiles(); tileNr++) {
						// drawing part
						try {
							ViewPort vp = tiles.getTileViewPort(viewPort, tileNr);
							drawTile(g, vp, cancel, tileNr);
						} catch (NoninvertibleTransformException e) {
							e.printStackTrace();
						}
					}
				}
			}else
				drawTile(g, viewPort, cancel, 0);
		}
		Runtime r = Runtime.getRuntime();
		long mem = r.totalMemory() - r.freeMemory();
		System.err.println("Memoria total: " + (mem / 1024) +"KB");
	}

	/**
	 * This is the method used to draw a tile in a WMS mosaic layer.
	 * @param tile Tile number to draw
	 */
	private void drawTile(Graphics2D g, ViewPort vp, Cancellable cancel, int tile) throws DriverException {

		// Compute the query geometry
		// 1. Check if it is within borders
		Rectangle2D extent = getFullExtent();
        if ((vp.getAdjustedExtent().getMinX() > extent.getMaxX()) ||
                (vp.getAdjustedExtent().getMinY() > extent.getMaxY()) ||
                (vp.getAdjustedExtent().getMaxX() < extent.getMinX()) ||
                (vp.getAdjustedExtent().getMaxY() < extent.getMinY())) {
            return;
        }

        // 2. Compute extent to be requested.
        Rectangle2D bBox = new Rectangle2D.Double();
        Rectangle2D.intersect(vp.getAdjustedExtent(), extent, bBox);

        // 3. Compute size in pixels
        double scalex = vp.getAffineTransform().getScaleX();
        double scaley = vp.getAffineTransform().getScaleY();
        int wImg = (int) Math.ceil(Math.abs(bBox.getWidth() * scalex) + 1);
        int hImg = (int) Math.ceil(Math.abs(bBox.getHeight() * scaley) + 1);
        Dimension sz = new Dimension(wImg, hImg);

        if ((wImg <= 0) || (hImg <= 0)) {
            return;
        }

		try {
			sz = new Dimension(wImg, hImg);
			Rectangle2D.intersect(vp.getAdjustedExtent(), extent, bBox);

			wcsStatus.setCoveraName( coverageName );
			wcsStatus.setExtent( bBox );
			wcsStatus.setFormat( format );
			wcsStatus.setHeight( hImg );
			wcsStatus.setWidth( wImg );
			wcsStatus.setSrs(srs);
			wcsStatus.setParameters( parameter );
			wcsStatus.setTime( time );
			wcsStatus.setOnlineResource((String) onlineResources.get("GetCoverage"));

			File f = getDriver().getCoverage(wcsStatus, new MyCancellable(cancel));
			if (f == null)
				return;
			String nameWordFile = f.getPath() + getExtensionWorldFile();
			com.iver.andami.Utilities.createTemp(nameWordFile, this.getDataWorldFile(bBox, sz));

			/*if(status!=null && firstLoad){
				status.applyStatus(this);
				firstLoad = false;
			}*/
			ViewPortData vpData = new ViewPortData(
				vp.getProjection(), new Extent(bBox), sz );
			vpData.setMat(vp.getAffineTransform());

			visualStatus.fileNames[tile] = f.getAbsolutePath();
			rasterProcess(g, vpData, f);

		} catch (ValidationException e) {
			UnknownResponseFormatExceptionType type =
				new UnknownResponseFormatExceptionType();
			type.setLayerName(getName());
			try {
				type.setDriverName(getDriver().getName());
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			type.setFormat(format);
			type.setHost(host);
			type.setProtocol("WCS");
			DriverException exception = new DriverException("unknown_response_format", type);
			throw exception;
//	azabala		throw new DriverException(PluginServices.getText(this, "unknown_response_format"), e);
		}
		catch (UnsupportedVersionException e) {
			UnsuportedProtocolVersionExceptionType type =
				new UnsuportedProtocolVersionExceptionType();
			type.setLayerName(getName());
			try {
				type.setDriverName(getDriver().getName());
			} catch (Exception ex){
			}
			type.setUrl(host);
			throw new DriverException(PluginServices.getText(this, "version_conflict"), e, type);

//	azabala		throw new DriverException(PluginServices.getText(this, "version_conflict"), e);
		} catch (IOException e) {
			ConnectionErrorExceptionType type = new ConnectionErrorExceptionType();
			type.setLayerName(getName());
			try {
				type.setDriverName(getDriver().getName());
			} catch (Exception e1) {
			}
			type.setHost(host);
			throw new DriverException(PluginServices.
					getText(this, "connect_error"), e, type);
		} catch (WCSException e) {
//azabala: la capturamos y la convertimos en DriverException
			WCSDriverExceptionType type = new WCSDriverExceptionType();
			type.setLayerName(getName());
			try {
				type.setDriverName(getDriver().getName());
			} catch (Exception e1) {
			}
			type.setWcsStatus(wcsStatus);
			this.setVisible(false);
			throw new DriverException("Error WCS", e,  type);

//            JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), e.getMessage());

        }//catch
	}

	/**
	 * Devuelve el FMapWMSDriver.
	 *
	 * @return FMapWMSDriver
	 *
	 * @throws IllegalStateException
	 * @throws ValidationException
	 * @throws UnsupportedVersionException
	 * @throws IOException
	 */
	private FMapWCSDriver getDriver() throws IllegalStateException, ValidationException, UnsupportedVersionException, IOException {
		if (wcs == null) {
			wcs = FMapWCSDriverFactory.getFMapDriverForURL(host);
		}
		return wcs;
	}

	/**
	 * Calcula el contenido del fichero de georreferenciacin de una imagen.
	 * @param bBox Tamao y posicin de la imagen (en coordenadas de usuario)
	 * @param sz Tamao de la imagen en pixeles.
	 * @return el 'WorldFile', como String.
	 * @throws IOException
	 */
	public String getDataWorldFile(Rectangle2D bBox, Dimension sz) throws IOException {
		StringBuffer data = new StringBuffer();
    	data.append((bBox.getMaxX() - bBox.getMinX())/(sz.getWidth() - 1)+"\n");
    	data.append("0.0\n");
    	data.append("0.0\n");
    	data.append("-"+(bBox.getMaxY() - bBox.getMinY())/(sz.getHeight() - 1)+"\n");
    	data.append(""+bBox.getMinX()+"\n");
    	data.append(""+bBox.getMaxY()+"\n");
    	return data.toString();
	}

	/**
	 * Dibuja una imagen usando PxRaster
	 * @param g	Graphics2D en el que hay que dibujar.
	 * @param vpData Parmetros de visualizacin
	 * @param file La imagen en cuestin.
	 */
	private void rasterProcess(Graphics2D g, ViewPortData vpData, File file)throws DriverException {

		//Creamos el PxRaster
		try{
			rasterFile = new GdalFile(vpData.getProjection(), file.getAbsolutePath());
		} catch (NotSupportedExtensionException e) {
			throw new DriverException("Open File Error");
		}
		raster = new PxRaster(rasterFile, null, rasterFile.getExtent());

		if(status!=null && firstLoad){
			status.applyStatus(this);
			firstLoad = false;
		}
		
		grid = new Grid(rasterFile);
		grid.addRenderizer(raster);
		
		visualStatus.rasterWidth += rasterFile.getWidth();
		visualStatus.rasterHeight += rasterFile.getHeight();
		if(raster.getExtent().getMin().getX() < visualStatus.rasterMinX)
			visualStatus.rasterMinX = raster.getExtent().getMin().getX();
		if(raster.getExtent().getMin().getY() < visualStatus.rasterMinY)
			visualStatus.rasterMinY = raster.getExtent().getMin().getY();
		if(raster.getExtent().getMax().getX() > visualStatus.rasterMaxX)
			visualStatus.rasterMaxX = raster.getExtent().getMax().getX();
		if(raster.getExtent().getMax().getY() > visualStatus.rasterMaxY)
			visualStatus.rasterMaxY = raster.getExtent().getMax().getY();
		
		//Recuperamos la pila de filtros si ya hubiese sido cargado antes
		if (this.filterStack!=null)
			raster.filterStack = this.filterStack;

		raster.setTransparency(false);

		//Asignamos transparencia y orden de bandas
		if (getTransparency() == -1 && !firstLoad);
		else
			raster.setTransparency(getTransparency());

		raster.setBand(GeoRasterFile.RED_BAND,rband);
		raster.setBand(GeoRasterFile.GREEN_BAND, gband);
		raster.setBand(GeoRasterFile.BLUE_BAND, bband);

		//Despues del primer pxRaster asignamos el stackManager guardado para los siguientes.
		//Con esto conseguimos asignar los cambios que se hayan producido desde el cuadro de
		//propiedades cuando creamos un nuevo pxRaster
		if (this.stackManager != null)
			raster.setStackManager(this.stackManager);

		if (visualStatus != null){
			visualStatus.bandCount = raster.getBandCount();
			visualStatus.dataType = raster.getDataType();
		}

		raster.draw(g, vpData);

		//En el primer pxRaster de una imagen obtenemos el Stack Manager para poder modificarlo
		//si queremos desde las propiedades

		if (this.stackManager == null)
			this.stackManager = raster.getStackManager();

		if (this.filterStack == null)
			this.filterStack = raster.filterStack;

		rasterFile.close();
	}

	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale, PrintRequestAttributeSet properties) throws DriverException {
		draw(null, g, viewPort, cancel, scale);
	}

	/**
	 * @return Returns the raster.
	 */
	public PxRaster getPxRaster() {
		return raster;
	}

	/**
	 * Returns the XMLEntity containing the necessary info for reproduce
	 * the layer.
	 *
	 * Devuelve el XMLEntity con la informacin necesaria para reproducir
	 * la capa.
	 *
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = super.getXMLEntity();

		xml.putProperty("wcs.host", getHost());
		xml.putProperty("wcs.fullExtent", StringUtilities.rect2String( fullExtent ));
		xml.putProperty("wcs.layerQuery", coverageName );
		xml.putProperty("wcs.format", format );
		xml.putProperty("wcs.srs", srs );
		xml.putProperty("wcs.time", time );
		xml.putProperty("wcs.parameter", parameter );
		xml.putProperty("wcs.coverageName", coverageName );
		xml.putProperty("wcs.maxResX", maxRes.getX());
		xml.putProperty("wcs.maxResY", maxRes.getY());
		if (status!=null)
			status.getXMLEntity(xml, true, this);
		else{
			status = new StatusLayerRaster();
			status.getXMLEntity(xml, true, this);
		}
		return xml;
	}

	/**
	 * Reproduces the layer from an XMLEntity.
	 *
	 * A partir del XMLEntity reproduce la capa.
	 *
 	 * @param xml XMLEntity
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public void setXMLEntity(XMLEntity xml) throws XMLException {
		super.setXMLEntity(xml);

		// host
		try {
			host = new URL(xml.getStringProperty("wcs.host"));
		} catch (MalformedURLException e) {
			throw new XMLException(e);
		}

		// full extent
		fullExtent = StringUtilities.string2Rect(xml.getStringProperty("wcs.fullExtent"));

		// coverageQuery
		coverageName = xml.getStringProperty("wcs.layerQuery");

		// format
		format = xml.getStringProperty("wcs.format");

		// srs
		srs = xml.getStringProperty("wcs.srs");

		// time
		time = xml.getStringProperty("wcs.time");

		// parameter
		parameter = xml.getStringProperty("wcs.parameter");

		// coverage name
		coverageName = xml.getStringProperty("wcs.coverageName");

		// max resolution
		if (xml.contains("wcs.maxRes"))
			maxRes = new Point2D.Double(xml.getDoubleProperty("wcs.maxRes"), xml.getDoubleProperty("wcs.maxRes"));
		else if (xml.contains("wcs.maxResX") && xml.contains("wcs.maxResY"))
			maxRes = new Point2D.Double(xml.getDoubleProperty("wcs.maxResX"), xml.getDoubleProperty("wcs.maxResY"));

		// OnlineResources
        if (xml.contains("onlineResources")) {
        	String[] operations = xml.getStringProperty("onlineResources").split("~##SEP1##~");
        	for (int i = 0; i < operations.length; i++) {
				String[] resources = operations[i].split("~##SEP2##~");
				if (resources.length==2 && resources[1]!="")
					onlineResources.put(resources[0], resources[1]);
			}
        }
		String claseStr = null;
		if (xml.contains("raster.class")) {
			claseStr = xml.getStringProperty("raster.class");
		}
		if (status!=null)
			status.setXMLEntity(xml, this);
		else {
			//Cuando cargamos un proyecto

			if(claseStr!=null && !claseStr.equals("")){
				try{
					Class clase = Class.forName(claseStr);
					Constructor constr = clase.getConstructor(null);
					status = (StatusRasterInterface)constr.newInstance(null);
					if(status!=null)
						status.setXMLEntity(xml, this);
				} catch(ClassNotFoundException exc) {
					exc.printStackTrace();
				} catch(InstantiationException exc) {
					exc.printStackTrace();
				} catch(IllegalAccessException exc) {
					exc.printStackTrace();
				} catch(NoSuchMethodException exc) {
					exc.printStackTrace();
				} catch(InvocationTargetException exc) {
					exc.printStackTrace();
				}
			}
		}
		firstLoad = true;
	}

	public void setCoverageName(String coverageName) {
		this.coverageName = coverageName;
	}

	public void setParameter(String parametersString) {
		this.parameter = parametersString;
	}

	public void setTime(String time) {
		this.time = time;
	}

	public void setSRS(String srs) {
		this.srs = srs;
	}

	public void setFormat(String format) {
		this.format = format;
	}


	/**
	 * Inserta el URL.
	 *
	 * @param host String.
	 * @throws MalformedURLException
	 */
	public void setHost(String host) {
		try {
			setHost(new URL(host));
		} catch (MalformedURLException e) {

		}
	}

	/**
	 * Inserta el URL.
	 *
	 * @param host URL.
	 */
	public void setHost(URL host) {
		this.host = host;
	}

	/**
	 * Sets the layer's full extent.
	 *
	 * Establece la extensin mxima de la capa.
	 *
	 * @param rect
	 */
	public void setFullExtent(Rectangle2D rect) {
		this.fullExtent = rect;
	}

	/**
	 * Devuelve el URL.
	 *
	 * @return URL.
	 */
	public URL getHost() {
		return host;
	}

	/**
	 * Remote source layers have a bunch of properties that are required for get them from
	 * the servers. This method supplies a hash table containing any needed field. This hash
	 * table may be used to let the client to connect to a server and restore a previously saved
	 * layer. So, the layer itself may not be saved to the disk since the actual saved
	 * info is just its properties.
	 *
	 * @return Returns a hash table containing all the required information for
	 * set up a wms layer
	 */
	public Hashtable getProperties(){
		Hashtable info = new Hashtable();
		info.put(   "name", coverageName);
		info.put(   "host", getHost());
		info.put(    "crs", srs);
		info.put( "format", format);
		String str = time;
		if (str==null)
			str = "";
		info.put(   "time", str);
		str = parameter;
		if (str==null)
			str = "";
		info.put("parameter", str);

		return info;
	}

	/**
	 * Obtiene la extensin del fichero de georreferenciacin
	 * @return String con la extensi�n del fichero de georreferenciaci�n dependiendo
	 * del valor del formato obtenido del servidor. Por defecto asignaremos un .wld
	 */
	private String getExtensionWorldFile(){
		String extWorldFile = ".wld";
    	if (format.equals("image/tif") || format.equals("image/tiff"))
    		extWorldFile = ".tfw";
    	if (format.equals("image/jpeg"))
    		extWorldFile = ".jpgw";
    	return extWorldFile;
	}

	public void setMaxResolution(Point2D maxResolution) {
		this.maxRes = maxResolution;
	}

	/**
	 * Gets the last open GeoRasterFile against the temp file received
	 *
	 * Obtiene el ltimo GeoRasterFile abierto sobre el temporal recibido
	 * @return
	 */
	public GeoRasterFile getGeoRasterFile(){
		return rasterFile;
	}


	/**
	 * Asignar el estado del raster
	 * @param status
	 */
	public void setStatus(StatusRasterInterface status){
		this.status = status;
	}

	/**
	 * Obtiene el estado del raster
	 * @return
	 */
	public StatusRasterInterface getStatus(){
		return this.status;
	}

	/**
	 * Gets the max resolution allowed by the coverage. Requesting a higher resolution
	 * than this value does not cause any error, but the info responsed is just an
	 * interpolation.
	 *
	 * In exchange for obtaining a greater file and without additional information,
	 * we can easily connect it at the View.
	 *
	 *
	 * Obtiene la resolucin mxima soportada por la cobertura. La peticin
	 * de una resolucin superior a la soportada no provoca ningn error, aunque
	 * la informacin obtenida slo es una mera interpolacin de informacin.
	 *
	 * A cambio de obtener un archivo mayor y sin informacin adicional, podemos
	 * fcilmente acoplarlo a la vista.
	 *
	 * @return double
	 */
	public Point2D getMaxResolution() {
		if (maxRes==null)
			maxRes = wcs.getMaxResolution(coverageName);
		return maxRes;
	}


	public void setDriver(FMapWCSDriver driver) {
		this.wcs = driver;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getTileSize()
	 */
	public int[] getTileSize() {
		int[] size = {maxTileDrawWidth, maxTileDrawHeight};
		return size;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#isTiled()
	 */
	public boolean isTiled() {
		return mustTileDraw;
	}
	
	/**
	 * Obtiene el tipo de dato de la capa raster
	 * @return Entero que representa el tipo de dato de la capa raster.
	 */
	public int getDataType() {
		return rasterFile.getDataType();
	}
	
	/**
	 * Obtiene el grid asociado al raster que hay cargado en ese momento
	 * @return grid
	 */
	public Grid getGrid(){
		return grid;
	}
}
