package com.iver.cit.gvsig.gui.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.Label;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;
import org.gvsig.gui.beans.filterPanel.filterQueryPanel.FilterQueryJPanel;
import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLElement;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.ComplexValue;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.IWindowListener;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.edition.EditableAdapter;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableAdapter;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.gui.filter.DefaultExpressionDataSource;
import com.iver.cit.gvsig.gui.filter.ExpressionDataSource;
import com.iver.cit.gvsig.gui.filter.ExpressionListener;
import com.iver.cit.gvsig.gui.filter.FilterException;
import com.iver.cit.gvsig.gui.panels.attributesTree.AttributesTreeTableModel;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.documents.table.ProjectTable;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;
import com.iver.cit.gvsig.project.documents.table.gui.Table;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.utiles.DefaultCharSet;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.exceptionHandling.ExceptionHandlingSupport;
import com.iver.utiles.exceptionHandling.ExceptionListener;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This will be the tab for add a filter to a WFS query.
 * This class gets the graphical interface from FilterQueryJPanel and add logic.
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class WFSFilterPanel extends FilterQueryJPanel implements IWindow, IWindowListener {
	private static Logger logger = Logger.getLogger(Table.class.getName());
	private WFSParamsPanel parent = null;
	private ArrayList expressionListeners = new ArrayList();
	private ExpressionDataSource model = null;
	private NumberFormat nf = NumberFormat.getNumberInstance();
	private ExceptionHandlingSupport exceptionHandlingSupport = new ExceptionHandlingSupport();
	private AttributesTreeTableModel attributesTreeTableModel;
	private boolean panelAsATabForWFSLayersLoad;
	private TreePath currentPath;
	private String featureName;
	
	/**
	 * This method initializes
	 *
	 */
	public WFSFilterPanel(WFSParamsPanel parent) {
		super();
		this.parent = parent;
		currentPath = null;
		featureName = null;

		// At beginning, the JList is disabled (and its set a particular color for user could knew it)
		super.getValuesJList().setEnabled(false);
		getValuesJList().setBackground(new Color(220, 220, 220));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#initialize()
	 */
	protected void initialize() {
		super.initialize();
		this.resizeHeight(380);

		defaultTreeModel = (DefaultTreeModel)fieldsJTree.getModel();
		
		this.addNewListeners();
		panelAsATabForWFSLayersLoad = true;		
	}
	
	/**
	 * Adds some more listener to the components of the panel
	 */
	private void addNewListeners() {
		
		// Enable "Apply" button when user changes the filter query
		txtExpression.getDocument().addDocumentListener(new DocumentListener() {
			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
			 */
			public void changedUpdate(DocumentEvent e) {
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
			 */
			public void insertUpdate(DocumentEvent e) {
				if (!panelAsATabForWFSLayersLoad)
					parent.isApplicable(true);
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
			 */
			public void removeUpdate(DocumentEvent e) {
				if (!panelAsATabForWFSLayersLoad)
					parent.isApplicable(true);
			}
		});
		
		// Listener for "fieldsJTree" 
		getFieldsJTree().addMouseListener(new MouseAdapter() {
			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				int row = fieldsJTree.getRowForLocation(e.getX(), e.getY());
				TreePath treePath = fieldsJTree.getPathForLocation(e.getX(), e.getY());

				if (row > -1) {
					switch (e.getClickCount()) {
						case 2:							
							putSymbolOfSelectedByMouseBranch(treePath);
							break;
					}
				}
			}
		});
		
		// Listener for "valuesJList"
		getValuesJList().addMouseListener(new MouseAdapter() {
			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				int index = getValuesJList().getSelectedIndex();
				
				// Avoids exception when no value is in the list
				if (index == -1)
					return;
				
				if (e.getClickCount() == 2) {
					Value valor = (Value) valuesListModel.getElementAt(index);
					
					if (getNodeOfCurrentPath().getEntityType().getName().compareTo("xs:string") == 0) {
						putSymbol("'" + valor.toString() + "'");
					}
					else {
						putSymbol(valor.toString());
					}					
				}
			}
		});
		
		// Listener for a branch of the tree selection
		getFieldsJTree().addTreeSelectionListener(new TreeSelectionListener() {
			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
			 */
			public void valueChanged(TreeSelectionEvent e) {
				if (!panelAsATabForWFSLayersLoad) {					
					DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
		
					if ((data != null) && (data.getData() != null)) {
//						setModel(data.getData());
						currentPath = e.getPath();
						fillValuesByPath(currentPath);
					}
				}
			}			
		});		
	}
	
	/**
	 * If there is a field selected, show its new values
	 */
	public void updateFieldValues() {
		if (currentPath != null) {
			
			DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
						
			if ((data != null) && (data.getData() != null)) {
				setModel(data.getData());				
				fillValuesByPath(currentPath);
				
				// Updates all tables that their data is about the changed view
				this.updateTablesThatHasNewData();
			}
		}
	}
	
	/**
	 * Updates all tables that their data is about the changed view
	 */
	private void updateTablesThatHasNewData() {
		boolean oneTimeNewDataToTableAdded = false;
		
		IWindow[] activeNoModalWindows = PluginServices.getMDIManager().getAllWindows();
		
		for (int i = 0; i < activeNoModalWindows.length; i++) {
			IWindow window = activeNoModalWindows[i];
			if (window instanceof Table) {
				Table table = (Table) window;
				
				int pos1 = featureName.indexOf(':');
				
				if ((pos1 >= 0) && (pos1 < featureName.length()))						
					featureName = featureName.substring(pos1 +1, featureName.length());
				
//					String featureOfTable = ((XMLElement)currentPath.getParentPath().getLastPathComponent()).getName();
					String featureOfTable = table.getModel().getName();
					int pos2 = featureOfTable.indexOf(':');
												
					if ((pos2 >= 0) && (pos2 < featureName.length()))
						featureOfTable = featureOfTable.substring(pos2 +1, featureOfTable.length());						
				
				if (featureName.trim().compareTo(featureOfTable.trim()) == 0) {
					// Only add the new data associated to the table one time
					if (oneTimeNewDataToTableAdded == false) {
						setNewDataToTable();
						oneTimeNewDataToTableAdded = true;
					}
				
					// Refresh the table with the new data
					table.refresh();
				}
			}
		}
	}
	
	/**
	 * This method is a modification of the "execute" method from the "ShowTable" class 
	 * @see com.iver.cit.gvsig.ShowTable#execute(String)
	 */
	private void setNewDataToTable() {
		View vista = (View) PluginServices.getMDIManager().getActiveWindow();
		FLayer[] actives = vista.getModel().getMapContext().getLayers().getActives();

		try {
			for (int i = 0; i < actives.length; i++) {
				if (actives[i] instanceof AlphanumericData) {
					AlphanumericData co = (AlphanumericData) actives[i];

					//SelectableDataSource dataSource;
					//dataSource = co.getRecordset();

					ProjectExtension ext = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);

					ProjectTable projectTable = ext.getProject().getTable(co);
					EditableAdapter ea=null;
					ReadableVectorial rv=((FLyrVect)actives[i]).getSource();
					if (rv instanceof VectorialEditableAdapter){
						ea=(EditableAdapter)((FLyrVect)actives[i]).getSource();
					}else{
						ea=new EditableAdapter();
						SelectableDataSource sds=((FLyrVect)actives[i]).getRecordset();
						ea.setOriginalDataSource(sds);
					}

					if (projectTable == null) {
						projectTable = ProjectFactory.createTable(PluginServices.getText(this, "Tabla_de_Atributos") + ": " + actives[i].getName(),
								ea);
						projectTable.setProjectDocumentFactory(new ProjectTableFactory());
						projectTable.setAssociatedTable(co);
						ext.getProject().addDocument(projectTable);
					}
					projectTable.setModel(ea);					
//					Table t = new Table();
//					t.setModel(projectTable);
//					PluginServices.getMDIManager().addWindow(t);
				}
			}
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            NotificationManager.addError(PluginServices.getText(this,"No_se_pudo_obtener_la_tabla_de_la_capa"), e);
        } catch (com.iver.cit.gvsig.fmap.DriverException e) {
			e.printStackTrace();
			NotificationManager.addError(PluginServices.getText(this,"No_se_pudo_obtener_la_tabla_de_la_capa"), e);
        }
	}

	/**
	 * Sets the value of the inner attribute: 'panelAsATabForWFSLayersLoad'
	 * 
	 * @param b A boolean value
	 */
	public void setWFSFilterPanelIsAsTabForWFSLayersLoad (boolean b) {
		this.panelAsATabForWFSLayersLoad = b;
		
		if (this.panelAsATabForWFSLayersLoad == true) {
			// At beginning, the JList is disabled (and its set a particular color for user could knew it)
			super.getValuesJList().setEnabled(false);
			super.getValuesJList().setBackground(new Color(220, 220, 220));
		}
		else {
			// Unselect the selected path in the tree (if there was any selected)
			if (this.currentPath != null) {
				this.currentPath = null;
				this.getFieldsJTree().removeSelectionPath(this.getFieldsJTree().getSelectionPath());
			}
			
			// Resets data loaded
			super.getValuesJList().setEnabled(true);
			super.getValuesJList().setBackground(Color.WHITE);
			
			this.getFieldsJTree().removeAll();
			this.getValuesJList().removeAll();
			
			// Updates data associated to view with the new layer data
			this.setNewDataToTable();
			
			// If theres is any table associated to the current view -> update that table/s
//			this.updateTablesThatHasNewData();
			
			// Reads that new data
			DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
			
			if ((data != null) && (data.getData() != null)) {
				setModel(data.getData());
//				currentPath = e.getPath();
//				fillValuesByPath(currentPath);
			}
		}
	}
	
	/**
	 * Gets the value of the inner attribute: 'panelAsATabForWFSLayersLoad'
	 * 
	 * @return A boolean value
	 */
	public boolean getWFSFilterPanelIsAsTabForWFSLayersLoad() {
		return this.panelAsATabForWFSLayersLoad;
	}
	
	/** 
	 * FIlls list with the values of selected field
	 * 
	 * @param treePath A path in the tree
	 */
	private void fillValuesByPath(TreePath treePath) {
		// Duplicates are removed
		TreeSet conjunto = new TreeSet(new Comparator() {
			public int compare(Object o1, Object o2) {
				if ((o1 != null) && (o2 != null)) {
					Value v2 = (Value) o2;
					Value v1 = (Value) o1;
					BooleanValue boolVal;
					
					try {
						boolVal = (BooleanValue) (v1.greater(v2));
						
						if (boolVal.getValue()) {
							return 1;
						}
						
						boolVal = (BooleanValue) (v1.less(v2));
						
						if (boolVal.getValue()) {
							return -1;
						}
					} catch (IncompatibleTypesException e) {
						throw new RuntimeException(e);
					}
				}
				
				return 0;
			}
		}); // For ordernation
		
		// Remove the previous items
		valuesListModel.clear();
		
		try {
			//Object root = treePath.getPath()[0];
			XMLElement element = ((XMLElement)treePath.getLastPathComponent());
			
			// Gets the values associated to the selected branch 
			switch (element.getEntityType().getType()) {
				case IXMLType.SIMPLE:
					
					if(element.getParentElement().getParentElement() == null) {					
						// Find the selected field and try to obtein values related
						for (int i = 0; i < model.getFieldCount(); i++) {					
							String name = model.getFieldName(i);
							
							// If we find the field (this means that are loaded its values and we can obtein them)
							if (name.equals(element.getName())) {						
								for (int j = 0; j < model.getRowCount(); j++) {					
									Value value = model.getFieldValue(j, i);
									
									if (value instanceof NullValue)
									    continue;
									
									if (!conjunto.contains(value)) {
										conjunto.add(value);
									}
								}
								
								break;
							}
						}
					}else{
						//create a vector with the parent names from the leaf until the root
						XMLElement parent = element.getParentElement();
						Vector parentNames = new Vector();
						parentNames.add(element.getName());
						while (parent != null){
							parentNames.add(parent.getName());
							parent = parent.getParentElement();							
						}
						
						//The field name (in the gvSIG table) is the second field name
						String fieldName = (String)parentNames.get(parentNames.size()-2);
						
						for (int i = 0; i < model.getFieldCount(); i++) {					
							String name = model.getFieldName(i);
							
							// If we find the field (this means that are loaded its values and we can obtein them)
							if (name.equals(fieldName)) {						
								for (int j = 0; j < model.getRowCount(); j++) {					
									Value value = model.getFieldValue(j, i);
															
									if (value instanceof NullValue)
									    continue;
									
									if (value instanceof ComplexValue){
										for (int k=parentNames.size()-3 ; k>=0 ; k--){
											ComplexValue complex = (ComplexValue)value;
											Value childValue = (Value)complex.get(parentNames.get(k));
											if (k==0){
												if (!conjunto.contains(childValue)) {
													conjunto.add(childValue);
												}
											}else{
												value = childValue;
											}
										}
									}
								}
								
								break;
							}
						}
					}
					break;
				case IXMLType.COMPLEX:
					break;
				default:
					// Do Nothing
			}
			
			// Add the values to the model of the graphic list
			Iterator it = conjunto.iterator();

			while (it.hasNext())
				valuesListModel.addElement(it.next());
		} catch (Exception e) {
			throwException(e);
		}
	}
	
	/**
	 * Puts the symbol of selected brach
	 * 
	 * @param mouseEvent A MouseEvent with information  of the selected branch
	 */
	public void putSymbolOfSelectedByMouseBranch(TreePath treePath) {
		// Sets the node selected
		if (treePath != null) {
			Object node = treePath.getLastPathComponent();
			if ((node != null) && (node instanceof XMLElement)) {
				XMLElement element = (XMLElement) node;
				XMLElement parent = element.getParentElement();
				String path = element.getName();
				while (parent.getParentElement() != null){
					path = parent.getName() + "/" + path;
					parent = parent.getParentElement();
				}
				putSymbol("\"" + path + "\"");
			}
		}
	}
	
	/**
	 * Gets the element that the 'currentPath' attribute aims
	 * 
	 * @return An XMLElement
	 */
	private XMLElement getNodeOfCurrentPath() {
		
		if (currentPath != null) {
			Object node = currentPath.getLastPathComponent();
			
			if ((node != null) && (node instanceof XMLElement)) {
//				XMLElement element = (XMLElement) node;
				return (XMLElement) node;
			}
		}
		
		return null;
	}
	
	/**
	 * Gets the query that will be send to the server
	 * @return SQL query (just the where part)
	 */
	public String getQuery(){
		try {
			return this.validateExpression();
		} catch (ParseException e) {			
			e.printStackTrace();
			return null;
		}		
	}
	
	/**
	 * Gets the filter expression from the user interface
	 */
	public String getFilterExpressionFromInterface() {
		return getTxtExpression().getText();
	}
	
	/**
	 * Writes the filter expression into the user interface
	 * @param filterExpression An string
	 */
	public void setFilterExpressionIntoInterface(String filterExpression){
		getTxtExpression().setText(filterExpression);
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean addExpressionListener(ExpressionListener arg0) {
		return expressionListeners.add(arg0);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean removeExpressionListener(ExpressionListener arg0) {
		return expressionListeners.remove(arg0);
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param o DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean removeExceptionListener(ExceptionListener o) {
		return exceptionHandlingSupport.removeExceptionListener(o);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	private void throwException(Throwable t) {
		exceptionHandlingSupport.throwException(t);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	public void setModel(ExpressionDataSource t) {
		try {
			model = t;
            model.start();
        } catch (DriverException e1) {
            NotificationManager.addError(e1.getMessage(), e1);
        }

        try {
        	int numberOfFields = model.getFieldCount();

        	if (numberOfFields > 0) {
        		Vector fields = new Vector(0, 1);
        		int j = 0;
        	
				for (int i = 0; i < numberOfFields; i++) {
					 Object field = model.getFieldName(i);
					
					if (field != null) {
						fields.add(field);
						j++;
					}
				}
			
				attributesTreeTableModel = new AttributesTreeTableModel(fields.toArray());			
        	}
		} catch (FilterException e) {
			throwException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param substring DOCUMENT ME!
	 * @param startingPos DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private int getIndex(String expresion, String substring, int startingPos) {
		int index = startingPos;

		do {
			index = expresion.indexOf(substring, index);
		} while ((StringUtilities.isBetweenSymbols(expresion, index, "\"")) &&
				(index != -1));

		return index;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param word DOCUMENT ME!
	 * @param translation DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateWord(String expresion, String word,
		String translation) throws ParseException {
		int booleanIndex = 0;
		int endIndex = 0;
		StringBuffer res = new StringBuffer();

		while ((booleanIndex = getIndex(expresion, word, booleanIndex)) != -1) {
			res.append(expresion.substring(endIndex, booleanIndex));
			endIndex = booleanIndex + word.length();
			booleanIndex++;
			res.append(translation);
		}

		if (endIndex < expresion.length()) {
			res.append(expresion.substring(endIndex));
		}

		return res.toString();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateDates(String expresion) throws ParseException {
		//Se obtiene el valor de la fecha
		String date = StringUtilities.substringDelimited(expresion, "Date(",
				")", 0);

		if (date == null) {
			return expresion;
		}

		//Se comprueba que no est entre comillas 
		int startIndex = expresion.indexOf(date);

		while (startIndex != -1) {
			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex - 5) +
					expresion.substring(startIndex).replaceFirst(date + "\\)",
						new Long((filterButtonsJPanel.getDateFormat().parse(date)).getTime()).toString());

			} else {
				startIndex += date.length();
			}
			
			if (date == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(date, startIndex);
		}

		return expresion;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	public String translateNumber(String expresion) throws ParseException {
		DefaultCharSet ss = new DefaultCharSet();
		ss.addInterval('0', '9');
		ss.addCharacter(',');
		ss.addCharacter('.');

		String number = StringUtilities.substringWithSymbols(expresion, ss, 0);

		if (number == null) {
			return expresion;
		}

		int startIndex = expresion.indexOf(number);

		while (startIndex != -1) {
			Number n = nf.parse(number);

			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex) +
					expresion.substring(startIndex).replaceFirst(number,
						n.toString());
			} else {
				startIndex += n.toString().length();
			}

			number = StringUtilities.substringWithSymbols(expresion, ss,
					startIndex);

			if (number == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(number, startIndex);
		}

		return expresion;
	}
	
	/**
	 * Encodes an string to ISO 8859_1 with each ' symbol converted to '' 
	 * 
	 * @param text An string started and finished with simple apostrophes
	 * @return An string started and finished with simple apostrophes
	 */
	private String translateString(String text) {
		// Encode to the string to ISO 8859_1 (the URL codification)
		try {
			
			// Ignore the first and last apostrophes
			if (text.length() > 2) {
				text = text.substring(1, text.length() -1);
				
				// Convert the string to ISO 8859_1 codification
				text = URLEncoder.encode(text, "8859_1");
				
				// Change '  (%27 code) to '' for the SQL parser (bebore sent the query)
				text = text.replaceAll("\\%27", "\\'\\'");
			}

		} catch (UnsupportedEncodingException e1) {
			e1.printStackTrace();
		}
		
		return "'" + text + "'";
	}
	
	/**
	 * DOCUMENT ME!
	 * (Queda por implementar, hay una versin reducida)
	 * 
	 * @return An string
	 * @throws ParseException An expection produced by the parser
	 */
	private String validateExpression() throws ParseException {
		String expression = txtExpression.getText().trim(); // Ignores the spaces at beginning and end of the chain of characters
		String result = new String("");
		
		// Encode each string of the query
		int index = 0;
		int lastIndex = 0;
		boolean endInnerLoop;
		// Encodes all inner strings to the equivalent codification in ISO-8859_1 with each ' symbol converted to ''
		while (index != -1) {
			index = expression.indexOf("'", index);
			
			// Add the parts of the chain of characters that not are string
			if (index == -1) {
				result += expression.substring(lastIndex, expression.length());
			}
			else {
				result += expression.substring(lastIndex, index).replaceAll(" [ ]+", " ");
			}
			
			lastIndex = index;
			endInnerLoop = false;
			
			// Tries to find each first apostrophe of each string of the query
			if ((index > 0) && (expression.charAt(index - 1) == ' ')) {
				index++;
				
				// Ignore all inner apostrophes and try to find the last of the string
				while (!endInnerLoop)  {
					index = expression.indexOf("'", index);
					index++;
					
					// If we haven't arrived to the finish of the string
					if (index != expression.length()) {
						if ((index == -1) || (expression.charAt(index) == ' ')) {
							result += translateString(expression.substring(lastIndex, index));
							endInnerLoop = true;
						}
					}
					else {
						result += translateString(expression.substring(lastIndex, index));
						endInnerLoop = true;
						index = -1; // Force to finish the external loop
					}
				}
				lastIndex = index;
			}
		}
		
		//Se transforman los nombres de los campos en las variables xix que analizarn
		//Se quitan los Date(fecha) y se mete la fecha correspondiente
		result = translateDates(result);
		result = translateNumber(result);
		result = translateWord(result, "true", "1");
		result = translateWord(result, "false", "0");

		logger.debug(result);

		return result;
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 * @deprecated In this moments its a reference of the old Validate Expression method
	 */
	private String oldValidateExpressionMethod() throws ParseException {
		String expression = txtExpression.getText();
//		HashSet variablesIndexes = new HashSet();
//
//		StringBuffer traducida = new StringBuffer();

		//Se transforman los nombres de los campos en las variables xix que analizarn
		//Se quitan los Date(fecha) y se mete la fecha correspondiente
		expression = translateDates(expression);
		expression = translateNumber(expression);
		expression = translateWord(expression, "true", "1");
		expression = translateWord(expression, "false", "0");

		String replacement;
		Pattern patron = Pattern.compile("[^<>!]=");
		Matcher m = patron.matcher(expression);
		int index = 0;

		while (m.find(index)) {
			index = m.start();
			replacement = expression.charAt(index) + "==";
			m.replaceFirst(replacement);
			index++;
		}

		expression = expression.replaceAll("[^<>!]=", "==");

		logger.debug(expression);

		return expression;
	}

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewActivated()
     */
    public void windowActivated() {
    }

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewClosed()
     */
    public void windowClosed() {
        try {
            model.stop();
        } catch (DriverException e) {
            NotificationManager.addError(e.getMessage(), e);
        }        
    }

	public WindowInfo getWindowInfo() {
		return null;
	}

	public void refresh(WFSLayerNode feature) {
		setFields(feature);
		featureName = feature.getTitle();
	}
	
	/**
	 * Sets Fields
	 *
	 * @param feature
	 */
	private void setFields(WFSLayerNode feature) {
		Vector fields = feature.getFields();
				
		this.resetFieldsAndValuesData();
		
		int numberOfFields = fields.size();
		
		if (numberOfFields > 0) {
			Vector fieldBranches = new Vector(0, 1);
			
			for (int i=0; i<fields.size(); i++) {
				XMLElement field = (XMLElement)fields.get(i);

				IXMLType type = field.getEntityType();
				
				if (type != null) {					
					
					switch (type.getType()) {
						case IXMLType.GML_GEOMETRY:
							break;
						case IXMLType.COMPLEX: case IXMLType.SIMPLE:
							fieldBranches.add(field);
							break;
					}
				}
			}
			
			attributesTreeTableModel = new AttributesTreeTableModel(fieldBranches.get(0));
			fieldsJTree.setModel(new AttributesTreeTableModel(fieldBranches.get(0), false));
		}
	}

	/**
	 * Resets the data of fields and their values of the current layer feature, and removes the branches of JTree
	 */
	private void resetFieldsAndValuesData() {
		fieldsJTree.setModel(new DefaultTreeModel(new DefaultMutableTreeNode()));
		txtExpression.setText("");
		((DefaultListModel)valuesJList.getModel()).removeAllElements();
	}
	
	
	/**
	 * @see WFSParamsPanel#isApplicable(boolean)
	 * 
	 * @param b A boolean value
	 */
	private void setApplicate(boolean b) {
		parent.isApplicable(b);
	}
}
