/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 */

package org.gvsig.remoteClient.arcims;

import org.gvsig.remoteClient.arcims.utils.ServiceInfoTags;

import java.io.IOException;

import java.net.ConnectException;


/**
 * Factory to create a proper handler based on the service type (ImageServer of FeatureServer)
 *
 */
public class ArcImsProtocolHandlerFactory {
    /**
     * Stablishes the type of Handler according with the service type
     * that will communicate with the server and
     * returns a convenient ProtocolHandler
     *
     * @param servicetype
     * @return instance of a client that is able to communicate with the host
     */
    public static ArcImsProtocolHandler negotiate(String servicetype)
        throws ConnectException, IOException {
        return createVersionDriver(servicetype);
    }

    /**
     * It creates an instance of a ArcImsProtocolHandler class.
     *
     * @param type of the driver to be created
     * @return ArcImsDriver
     */
    private static ArcImsProtocolHandler createVersionDriver(String type) {
        try {
            Class driver;

            //           version = version.replace('.', '_');
            //           driver = Class.forName("org.gvsig.remoteClient.arcims.arcims_"+version+".ArcImsProtocolHandler" + version);
            if (type.equals(ServiceInfoTags.vIMAGESERVICE)) {
                driver = Class.forName(
                        "org.gvsig.remoteClient.arcims.ArcImsProtImageHandler");
            } else {
                driver = Class.forName(
                        "org.gvsig.remoteClient.arcims.ArcImsProtFeatureHandler");
            }

            return (ArcImsProtocolHandler) driver.newInstance();
        } catch (Exception e) {
            e.printStackTrace();

            //throw new Exception("ArcImsDriverFactory. Unknown driver version " + e);
            return null;
        }
    }
}
