/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Parser bsico XML para las ventanas de propiedades de los drivers de escritura.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class CXMLParser {
    private String XML;

    /**
     * Constructor
     * @param fXML        Texto en formato XML
     */
    public CXMLParser(String fXML) {
        XML = fXML;
    }

    /**
     * Obtiene texto XML
     * @return        texto XML
     */
    public String getXML() {
        return this.XML;
    }

    /**
     * Obtiene el valor del tag
     * @param tag        Tag del cual se quiere obtener el valor
     * @param bDeleteValue        Si es true elimina el tag y si es false lo mantiene
     * @return        Devuelve el contenido del tag
     */
    public String getValue(String tag, boolean deleteValue) {
        XML = XML.trim();

        String startTag = new String("<" + tag);
        String endTag = new String("</" + tag + ">");

        int iStart = XML.indexOf(startTag);
        int endOpenTag = XML.indexOf(">");

        while ((endOpenTag < iStart) && (endOpenTag != -1))
            endOpenTag = XML.indexOf(">", endOpenTag + 1);

        iStart = endOpenTag + 1;

        if (iStart == 0) {
            return "";
        }

        int iEnd = XML.indexOf(endTag);

        if (iEnd == 0) {
            return "";
        }

        String res;

        try {
            res = XML.substring(iStart, iEnd);

            if (deleteValue) {
                int start = XML.indexOf(startTag);
                int end = XML.indexOf(">");

                while ((end < start) && (end != -1))
                    end = XML.indexOf(">", end + 1);

                String subcadena = XML.substring(start, end + 1);
                Pattern p = Pattern.compile(subcadena);
                Matcher m = p.matcher(XML);
                XML = m.replaceFirst("");
                p = Pattern.compile(endTag);
                m = p.matcher(XML);
                XML = m.replaceFirst("");
                XML = XML.trim();

                if (!XML.startsWith("<")) {
                    if (XML.indexOf("<") == -1) {
                        XML = new String("");
                    } else {
                        XML = XML.substring(XML.indexOf("<"), XML.length());
                    }
                }
            }
        } catch (StringIndexOutOfBoundsException e) {
            return "";
        }

        return res;
    }

    /**
     * Obtiene un atributo de la primera aparicin de un  tag concreto
     * @param tag        Tag del cual quiere obtenerse un atributo
     * @param att        Atributo del cual se desea el valor
     * @return
     */
    public String getAttr(String tag, String att) {
        XML = XML.trim();

        int start = XML.indexOf(tag) + tag.length();
        int end = XML.indexOf(">");

        while ((end < start) && (end != -1))
            end = XML.indexOf(">", end + 1);

        String subcadena = XML.substring(start, end);
        start = subcadena.indexOf(att) + att.length() + 1;

        if (subcadena.indexOf(att) != -1) {
            end = subcadena.indexOf("\"", start + 1);

            //System.out.println(start+" "+end);
            if (end == -1) {
                end = subcadena.length();
            }

            return subcadena.substring(start + 1, end);
        } else {
            return new String("");
        }
    }

    /**
     * Obtiene el nombre del siguiente tag XML
     * @return        Nombre del siguiente tag
     */
    public String getTag() {
        XML = XML.trim();

        int start = XML.indexOf("<");

        if (start == -1) {
            return new String("");
        }

        int end = XML.indexOf(" ");

        if ((end > XML.indexOf(">")) || (end == -1)) {
            end = XML.indexOf(">");
        }

        if (end == -1) {
            return new String("");
        }

        return XML.substring(start + 1, end);
    }

    /**
     * Dado un nombre de un tag devuelve el nmero de estos en el XML
     * @param tag        Nombre del tag a contar
     * @return        Nmero de tags coincidentes
     */
    public int getNumTags(String tag) {
        int numTags = 0;
        int index = XML.indexOf("<" + tag);

        while (index != -1) {
            numTags++;
            index = XML.indexOf("<" + tag, index + 1);
        }

        return numTags;
    }
}
