/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.core;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;

import javax.print.attribute.PrintRequestAttributeSet;

import com.iver.utiles.IPersistance;
import com.iver.utiles.XMLEntity;

public interface ISymbol extends IPersistance {
	public final static int CHARACTER_MARKER = 1;
	public final static int SIMPLE_FILL = 2;
	public static final int MARKER_FILL = 3;
	ISymbol getSymbolForSelection();

	void draw(Graphics2D g, AffineTransform affineTransform, FShape shp);

	// In the future, we may need something like that, but now if
	// I put it, we should change the rendering process, and it will
	// be slowly.
	// El problema es que antes de renderizar algo, tenemos que calcular
	// cmo va a quedar en pantalla, para saber su bounding box real y si
	// hay que dibujarlo o no. Pasa con los textos, pero en general
	// con cualquier smbolo de tipo puntual, tambin. En menor medida con
	// lneas y polgonos.
	//
	// IDEA!!!:
	// Para no depender de leer un shape y pedirle al smbolo que toca
	// el bounding box para ver si hay que dibujarlo, se puede hacer una
	// pasada previa por la leyenda y ver los pixels que aade al bounding
	// box. (Se coge un punto, una lnea o un polgono, y se calcula
	// su bounding box para esa escala. Si es mayor que el original del
	// shape, se calculan los pixels. Luego se pasan esos pixels a
	// coordenadas de mundo real, y se le suman al visibleExtent para que
	// se dibujen los trozos de texto que no se pintan, etc, etc.
	int getPixExtentPlus(Graphics2D g, AffineTransform affineTransform, Shape shp);
	/**
	 * Devuelve el rgb del smbolo cuando se dibuja como un punto.
	 *
	 * @return rgb del smbolo.
	 */
	public int getOnePointRgb();

	/**
	 * Use it to specify how do you want to write the symbol in .gvp project
	 * @return
	 */
	XMLEntity getXMLEntity();

	/**
	 * The description is a human-readable text used to
	 * @return description of this symbol.
	 */
	String getDescription();

	/**
	 * Tells whether the shape of the symbol will be drawn or not.
	 *
	 * @return <b>true</b> if Shape must be drawn. Useful if you are labelling
	 */
	boolean isShapeVisible();

	void setDescription(String string);

	/**
	 * @return FSymbol constants. I think it is better to use isSuitableFor
	 */
	int getSymbolType();

	/**
	 * True if this symbol is ok for the geometry. For example, a FillSymbol will
	 * be suitable for a Polygon.
	 * @param geom
	 * @return
	 */
	boolean isSuitableFor(IGeometry geom);

	/**
	 * Useful to render the symbol inside the TOC, or inside little
	 * rectangles. For example, think about rendering a Label with size
	 * in meters => You will need to specify a size in pixels.
	 * Of course, you can also to choose to render a prepared image, etc.
	 * @param g2
	 * @param scaleInstance
	 * @param r
	 */
	void drawInsideRectangle(Graphics2D g, AffineTransform scaleInstance, Rectangle r);

	void setPrintingProperties(PrintRequestAttributeSet printProperties);


}


