/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;
import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.IPersistance;

/**
 * Implementacin de las caractersticas de alto nivel de las capas:
 * visibilidad, activacin, nombre, ...
 */
public abstract class FLyrDefault implements FLayer {
	// private PropertyChangeSupport lnkPropertyChangeSupport;
	private static Logger logger = Logger.getLogger(FLyrDefault.class);

	/** Path de la capa en el arbol de capas */
	private FLayers parentLayer = null;

	private FLayers virtualLayers = null;

	private FLyrText layerText = null;

	private String name;

	private IProjection projection;

//	private boolean visible = true;
//
//	private boolean active;

	private int transparency = 0;

	private ICoordTrans ct;

	private double minScale = -1; // -1 indica que no se usa

	private double maxScale = -1;

//	private boolean isInTOC = true;

	protected ArrayList layerListeners = new ArrayList();

	private Strategy privateStrategy = null;

//	private boolean isediting;

	private Hashtable properties = new Hashtable();

//	private boolean bCacheDrawnLayers;

	private BufferedImage cacheImageDrawnLayers = null;

//	private boolean bDirty;

//	private boolean available = true;

	//by default, all is active, visible and avalaible
	private FLayerStatus status = new FLayerStatus();

	public Object getProperty(Object key) {
		return properties.get(key);
	}

	public void setProperty(Object key, Object val) {
		properties.put(key, val);
	}

	public Map getExtendedProperties() {
		return properties;
	}
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setActive(boolean)
	 */
	public void setActive(boolean selected) {
		//active = selected;
		status.active = selected;
		callActivationChanged(LayerEvent.createActivationChangedEvent(this,
				"active"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#isActive()
	 */
	public boolean isActive() {
//		return active;
		return status.active;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setName(java.lang.String)
	 */
	public void setName(String name) {
		this.name = name;
		callNameChanged(LayerEvent.createNameChangedEvent(this, "name"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getName()
	 */
	public String getName() {
		return name;
	}

	/*
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#load()
	 */
	public void load() throws DriverIOException {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setVisible(boolean)
	 */
	public void setVisible(boolean visibility) {
//		visible = visibility;
		boolean changed = status.visible != visibility;
		status.visible = visibility;
		setDirty(true);
		if (changed){
			if (this.getMapContext() != null){
				this.getMapContext().clearAllCachingImageDrawnLayers();
			}
		}
		callVisibilityChanged(LayerEvent.createVisibilityChangedEvent(this,
				"visible"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#isVisible()
	 */
	public boolean isVisible() {
//		return visible && this.available;
		return status.visible && status.available;
	}

	/**
	 * Devuelve la capa padre de la actual.
	 *
	 * @return FLayers padre.
	 */
	public FLayers getParentLayer() {
		return parentLayer;
	}

	/**
	 * Inserta la capa padre.
	 *
	 * @param root
	 *            capa padre.
	 */
	public void setParentLayer(FLayers root) {
		this.parentLayer = root;
	}

	/**
	 * Inserta una proyeccin.
	 *
	 * @param proj
	 *            Proyeccin.
	 */
	public void setProjection(IProjection proj) {
		projection = proj;
		// Comprobar que la proyeccin es la misma que la de FMap
		// Si no lo es, es una capa que est reproyectada al vuelo
		if ((proj != null) && (getMapContext() != null))
			if (proj != getMapContext().getProjection()) {
				ICoordTrans ct = proj.getCT(getMapContext().getProjection());
				setCoordTrans(ct);
				logger.debug("Cambio proyeccin: FMap con "
						+ getMapContext().getProjection().getAbrev() + " y capa "
						+ getName() + " con " + proj.getAbrev());
			}
	}

	/**
	 * @see org.cresques.geo.Projected#getProjection()
	 */
	public IProjection getProjection() {
		return projection;
	}

	/**
	 * @see org.cresques.geo.Projected#reProject(org.cresques.cts.ICoordTrans)
	 */
	public void reProject(ICoordTrans arg0) {
	}

	/**
	 * Devuelve el nivel de transparencia de la capa.
	 *
	 * @return Entero que representa el nivel de transparencia.
	 */
	public int getTransparency() {
		return transparency;
	}

	/**
	 * Inserta el nivel de transparencia de la capa.
	 *
	 * @param trans
	 *            Nivel de transparencia.
	 */
	public void setTransparency(int trans) {
		transparency = trans;
		setDirty(true);
	}

	/**
	 * Devuelve el XMLEntity a partir del objeto.
	 *
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className", this.getClass().getName());

		if (this instanceof FLayers) {
		}

//		xml.putProperty("active", active);
		xml.putProperty("active", status.active);
		xml.putProperty("name", name);
		xml.putProperty("minScale", minScale);
		xml.putProperty("maxScale", maxScale);

		// TODO xml.addChild(parentLayer.getXMLEntity());
//		xml.putProperty("visible", visible);
		xml.putProperty("visible", status.visible);
		if (projection != null) {
			xml.putProperty("proj", projection.getAbrev());
		}
		xml.putProperty("transparency", transparency);
//		xml.putProperty("isInTOC", isInTOC);
		xml.putProperty("isInTOC", status.inTOC);

		// persist Properties hashTable

		Set keyset = properties.keySet();
		

		int numProperties = 0;
		Iterator keyitr = keyset.iterator();
	    while (keyitr.hasNext()) {
	      String propName = (String)keyitr.next();
	      Object obj = properties.get(propName);
	      if (obj instanceof IPersistance)
	      {
	    	  IPersistance persistObj = (IPersistance)obj;
	          XMLEntity xmlPropObj = persistObj.getXMLEntity();
              // make sure the node contains the class name
	          if (!xmlPropObj.contains("className")) {
	        	  try {
	        		  String propClassName = persistObj.getClassName();
		        	  System.out.println("PROP CLASS NAME "+propClassName);
		        	  xmlPropObj.putProperty("className", propClassName);
	        	  } catch (Exception e) {
	        		  e.printStackTrace();
	        	  }
	          }
	          xmlPropObj.putProperty("layerPropertyName", propName);
	          xml.addChild(xmlPropObj);
	          numProperties++;
	      } else if (obj instanceof String) {
	          XMLEntity xmlPropObj = new XMLEntity();
        	  xmlPropObj.putProperty("className", String.class.getName());
        	  xmlPropObj.putProperty("value",(String)obj);
	          xmlPropObj.putProperty("layerPropertyName", propName);
	          xml.addChild(xmlPropObj);
	          numProperties++;
	      }
	    }
	    xml.putProperty("numProperties", numProperties);

		return xml;
	}

	/*
	 * Inserta los valores de los atributos del XMLEntity al objeto.
	 *
	 * @param xml XMLEntity.
	 *
	 * @throws XMLException @throws DriverException @throws DriverIOException
	 *
	 * public void setXMLEntity03(XMLEntity xml) throws XMLException { active =
	 * xml.getBooleanProperty("active"); name = xml.getStringProperty("name");
	 * minScale=xml.getDoubleProperty("minScale");
	 * maxScale=xml.getDoubleProperty("maxScale"); visible =
	 * xml.getBooleanProperty("visible"); if (xml.contains("proj")) {
	 * setProjection(ProjectionPool.get(xml.getStringProperty("proj"))); } if
	 * (xml.contains("transparency")) transparency =
	 * xml.getIntProperty("transparency"); }
	 */

	/**
	 * Inserta los valores de los atributos del XMLEntity al objeto.
	 *
	 * @param xml
	 *            XMLEntity.
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public void setXMLEntity(XMLEntity xml) throws XMLException {
//		active = xml.getBooleanProperty("active");
		status.active = xml.getBooleanProperty("active");
		name = xml.getStringProperty("name");
		minScale = xml.getDoubleProperty("minScale");
		maxScale = xml.getDoubleProperty("maxScale");
//		visible = xml.getBooleanProperty("visible");
		status.visible = xml.getBooleanProperty("visible");
		if (xml.contains("proj")) {
			setProjection(CRSFactory.getCRS(xml.getStringProperty("proj")));
		}
		if (xml.contains("transparency"))
			transparency = xml.getIntProperty("transparency");
		if (xml.contains("isInTOC"))
//			isInTOC = xml.getBooleanProperty("isInTOC");
			status.inTOC = xml.getBooleanProperty("isInTOC");

        // recreate Properties hashTable

		if (xml.contains("numProperties")) {
			int numProps = xml.getIntProperty("numProperties");
			Object obj= null;
			IPersistance objPersist;
            for (int iProp=0; iProp<numProps; iProp++) {
        	    XMLEntity xmlProp = xml.getChild(0);
        	    try {
        	    	String className = xmlProp.getStringProperty("className");
        	    	if (className.equals(String.class.getName())) {
        	    		obj = xmlProp.getStringProperty("value");
        	    	} else {
	        	        Class classProp = Class.forName(className);
		                obj = classProp.newInstance();
	    	            objPersist = (IPersistance)obj;
	        	        objPersist.setXMLEntity(xmlProp);
	                }
	                String propName = xmlProp.getStringProperty("layerPropertyName");
	                properties.put(propName, obj);
        		} catch (Exception e) {
        			continue;
        		}
        		// remove Properties children to avoid breaking layers' XML reading logic
        		xml.removeChild(0);
            }
		}
	}

	/**
	 * Inserta los valores de los atributos del XMLEntity al objeto.
	 *
	 * @param xml
	 *            XMLEntity.
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public void setXMLEntity03(XMLEntity xml) throws XMLException {
//		active = xml.getBooleanProperty("active");
		status.active = xml.getBooleanProperty("active");
		name = xml.getStringProperty("name");
		minScale = xml.getDoubleProperty("minScale");
		maxScale = xml.getDoubleProperty("maxScale");
//		visible = xml.getBooleanProperty("visible");
		status.visible = xml.getBooleanProperty("visible");
		if (xml.contains("proj")) {
			setProjection(CRSFactory.getCRS(xml.getStringProperty("proj")));
		}
		if (xml.contains("transparency"))
			transparency = xml.getIntProperty("transparency");
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMapContext()
	 */
	public MapContext getMapContext() {
		if (getParentLayer() != null) {
			return getParentLayer().getMapContext();
		} else {
			return null;
		}
	}

	/**
	 * Aade a la lista de listener un nuevo LayerListener.
	 *
	 * @param o
	 *            LayerListener.
	 *
	 * @return boolean.
	 */
	public boolean addLayerListener(LayerListener o) {
		if (layerListeners.contains(o))
			return false;
		return layerListeners.add(o);
	}
	public LayerListener[] getLayerListeners() {
		return (LayerListener[])layerListeners.toArray(new LayerListener[0]);
	}
	/**
	 * Borra de la lista de listeners el que se pasa como parmetro.
	 *
	 * @param o
	 *            LayerListener a borrar.
	 *
	 * @return True si ha sido correcto el borrado del Listener.
	 */
	public boolean removeLayerListener(LayerListener o) {
		return layerListeners.remove(o);
	}

	/**
	 * Llamada al metodo nameChanged de los Listeners dados de alta.
	 *
	 * @param e
	 *            LayerEvent.
	 */
	private void callNameChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.nameChanged(e);
		}
	}

	/**
	 * Llamada al mtodo visibilityChanged de los Listeners.
	 *
	 * @param e
	 *            LayerEvent.
	 */
	private void callVisibilityChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.visibilityChanged(e);
		}
	}

	/**
	 * Llamada al mtodo activationChanged de los Listener.
	 *
	 * @param e
	 *            LayerEvent.
	 */
	private void callActivationChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.activationChanged(e);
		}
	}

	/**
	 * Devuelve las capas virtuales.
	 *
	 * @return FLayers.
	 */
	public FLayers getVirtualLayers() {
		return virtualLayers;
	}

	/**
	 * Inserta las capas virtuales.
	 *
	 * @param virtualLayers
	 *            FLayers.
	 */
	public void setVirtualLayers(FLayers virtualLayers) {
		this.virtualLayers = virtualLayers;
	}

	/**
	 * Devuelve la capa de texto.
	 *
	 * @return capa de texto.
	 */
	public FLyrText getLayerText() {
		return layerText;
	}

	/**
	 * Inserta la capa de texto.
	 *
	 * @param layerText
	 *            Capa de texto.
	 */
	public void setLayerText(FLyrText layerText) {
		this.layerText = layerText;
	}

	/**
	 * Inserta la Transformacin de coordenadas.
	 *
	 * @param ct
	 *            Transformacin de coordenadas.
	 */
	public void setCoordTrans(ICoordTrans ct) {
		this.ct = ct;
	}

	/**
	 * Devuelve las transformacin de coordenadas.
	 *
	 * @return ct.
	 */
	public ICoordTrans getCoordTrans() {
		return ct;
	}

	/**
	 * Mtodo que es llamado por Flayers para notificar a la capa que va a ser
	 * aadida. Esta previa notificacin es util para las capas que necesitan
	 * hacer algo antes de ser aadida. Por ejemplo, el raster necesita volver a
	 * abrir el fichero que ha podido ser cerrado con anterioridad. Si no se
	 * redefine este mtodo no se har nada ya que este es vacio.
	 */
	public void wakeUp() {
	}

	public double getMinScale() {
		return minScale;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMaxScale()
	 */
	public double getMaxScale() {
		return maxScale;
	}

	public void setMinScale(double minScale) {
		this.minScale = minScale;
	}

	public void setMaxScale(double maxScale) {
		this.maxScale = maxScale;
	}

	public boolean isWithinScale(double scale) {

		boolean bVisible = true;
		if (getMinScale() != -1) {
			if (scale < getMinScale())
				bVisible = false;
		}
		if (getMaxScale() != -1) {
			if (scale > getMaxScale())
				bVisible = false;
		}

		return bVisible;
	}

	public Strategy getStrategy() {
		return privateStrategy;
	}

	public void setStrategy(Strategy s) {
		privateStrategy = s;
	}

	public void setEditing(boolean b) throws EditionException {
//		isediting = b;
		status.editing = b;
		setDirty(true);
		setCachingDrawnLayers(b);
	}

	protected void callEditionChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.editionChanged(e);
		}
	}

	public boolean isEditing() {
//		return isediting;
		return status.editing;
	}

	public ImageIcon getTocImageIcon() {
		return null;
	}

	public boolean isInTOC() {
//		return isInTOC;
		return status.inTOC;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#isCachingDrawnLayers()
	 */
	public boolean isCachingDrawnLayers() {
//		return bCacheDrawnLayers;
		return status.cacheDrawnLayers;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setCachingDrawnLayers(boolean)
	 */
	public void setCachingDrawnLayers(boolean bCacheDrawnLayers) {
//		this.bCacheDrawnLayers = bCacheDrawnLayers;
		status.cacheDrawnLayers = bCacheDrawnLayers;
		if (status.cacheDrawnLayers == false)
			this.cacheImageDrawnLayers = null;
	}

	public BufferedImage getCacheImageDrawnLayers() {
		return cacheImageDrawnLayers;
	}

	public void setCacheImageDrawnLayers(BufferedImage cacheImageDrawnLayers) {
		this.cacheImageDrawnLayers = cacheImageDrawnLayers;
	}

	public boolean isDirty() {
		return status.dirty;
	}

	public void setDirty(boolean dirty) {
		status.dirty = dirty;
	}

	public boolean isAvailable() {
		return status.available;
	}

	public void setAvailable(boolean available) {
		status.available = available;
	}

	public void reload() throws DriverIOException {
		this.setAvailable(true);
	}

	/**
	 * Returns the status of the layer
	 */
	public FLayerStatus getFLayerStatus(){
		return status;
	}
	/**
	 * Sets the status of the layer
	 * @param status
	 */
	public void setFLayerStatus(FLayerStatus status){
		this.status = status;
	}

	/*
	 * This stuff is to save error's info that causes
	 * unavailable status.
	 * */

	public boolean isOk(){
		return status.isOk();
	}

	public int getNumErrors(){
		return status.getNumErrors();
	}

	public DriverException getError(int i){
		return status.getError(i);
	}

	public List getErrors(){
		return status.getErrors();
	}

	public void addError(DriverException error){
		status.addLayerError(error);
	}

	public boolean visibleRequired() {
		return status.visible;
	}

	public String getInfoString() {
		return null;
	}

	public boolean isWritable() {
		return status.writable;
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#cloneLayer()
	 */
	public FLayer cloneLayer() throws Exception {
		return this;
	}



}
