/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import com.iver.cit.gvsig.fmap.FMapEvent;


/**
 * Evento sobre las capas.
 *
 * @author Vicente Caballero Navarro
 */
public class LayerCollectionEvent extends FMapEvent {
	private final static int LAYER_ADDED = 0;
	private final static int LAYER_REMOVED = 2;
	private final static int LAYER_ADDING = 3;
	private final static int LAYER_REMOVING = 5;
	private final static int LAYER_ACTIVATION_CHANGED = 6;
	private final static int LAYER_VISIBILITY_CHANGED = 7;
	
	private FLayers layers;
	private FLayer affected;
	public static LayerCollectionEvent createLayerAddedEvent(FLayer lyr){
		return new LayerCollectionEvent(lyr, LAYER_ADDED);
	}

	public static LayerCollectionEvent createLayerRemovedEvent(FLayer lyr){
		return new LayerCollectionEvent(lyr, LAYER_REMOVED);
	}

	public static LayerCollectionEvent createLayerAddingEvent(FLayer lyr){
		return new LayerCollectionEvent(lyr, LAYER_ADDING);
	}

	public static LayerCollectionEvent createLayerRemovingEvent(FLayer lyr){
		return new LayerCollectionEvent(lyr, LAYER_REMOVING);
	}

	public static LayerCollectionEvent createLayerActivationEvent(FLayer lyr){
		return new LayerCollectionEvent(lyr, LAYER_ACTIVATION_CHANGED);
	}

	public static LayerCollectionEvent createLayerVisibilityEvent(FLayer lyr){
		return new LayerCollectionEvent(lyr, LAYER_VISIBILITY_CHANGED);
	}
	
	/**
	 * Crea un nuevo LayerCollectionEvent.
	 *
	 * @param lyr FLayers.
	 */
	protected LayerCollectionEvent(FLayer lyr, int eventType) {
		layers = lyr.getParentLayer();
		affected = lyr;
		setEventType(eventType);
	}

	/**
	 * Devuelve las FLayers.
	 *
	 * @return FLayers.
	 */
	public FLayers getLayers() {
		return layers;
	}

	/**
	 * Devuelve las FLayers afectadas por el cambio.
	 *
	 * @return FLayers.
	 */
	public FLayer getAffectedLayer() {
		return affected;
	}
}
