/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers.layerOperations;

import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionListener;


/**
 * Interfaz que implementan todas las capas que pueden ser coleccin de otras.
 */
public interface LayerCollection {
	/**
	 * Aade un listener de los eventos de la coleccin de capas
	 *
	 * @param listener LayerCollectionListener.
	 */
	public void addLayerCollectionListener(LayerCollectionListener listener);

	/**
	 * ELimina un listener de los eventos de la coleccin de capas
	 *
	 * @param listener LayerCollectionListener.
	 */
	public void removeLayerCollectionListener(LayerCollectionListener listener);

	/**
	 * Aade una capa al final de la lista de subcapas
	 *
	 * @param layer FLayer.
	 *
	 * @throws CancelationException
	 */
	public void addLayer(FLayer layer) throws CancelationException;

	/**
	 * Mueve una capa de la posicin from a la posicin to. (Se supone que
	 * estn dentro del mismo nivel). Para mover entre niveles, usar otro
	 * mtodo (por hacer)
	 *
	 * @param from origen.
	 * @param to destino.
	 *
	 * @throws CancelationException
	 */
	public void moveTo(int from, int to) throws CancelationException;

	/**
	 * Borra un FLayer a partir del objeto FLayer que se pasa como parmetro.
	 *
	 * @param lyr Layer a borrar.
	 *
	 * @throws CancelationException
	 */
	public void removeLayer(FLayer lyr) throws CancelationException;

	/**
	 * Borra una FLayer del vector a partir del identificador.
	 *
	 * @param idLayer
	 */
	public void removeLayer(int idLayer);

	/**
	 * Mtodo de conveniencia para borrar una capa con ese nombre
	 *
	 * @param layerName
	 */
	public void removeLayer(String layerName);

	/**
	 * Obtiene el array de capas visibles que penden del arbol cuya raz es
	 * este nodo
	 *
	 * @return Array capas visibles.
	 */
	public FLayer[] getVisibles();

	/**
	 * Obtiene el array de capas activas en el subrbol que pende de este nodo
	 *
	 * @return Array capas activas.
	 */
	public FLayer[] getActives();

	/**
	 * Obtiene el hijo i-simo directo de esta coleccin
	 *
	 * @param index ndice de la capa.
	 *
	 * @return FLayer.
	 */
	public FLayer getLayer(int index);

	/**
	 * Mtodo de conveniencia para buscar una capa por su nombre Es conveniente
	 * usar getLayer(int) siempre que se pueda, es ms directo.
	 *
	 * @param layerName Nombre de la capa.
	 *
	 * @return FLayer.
	 */
	public FLayer getLayer(String layerName);

	/**
	 * Obtiene el nmero de capas que hay en su mismo nivel Es decir, no cuenta
	 * las subcapas de un FLayers
	 *
	 * @return Nmero de capas.
	 */
	public int getLayersCount();

	/**
	 * Pone todas las capas a activas o inactivas.
	 *
	 * @param active boolean.
	 */
	public void setAllActives(boolean active);

	/**
	 * Pone todas las capas a visibles o invisibles.
	 *
	 * @param visible boolean.
	 */
	public void setAllVisibles(boolean visible);
}
