/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

/**
 * Interfaz con los eventos que puede realizar una leyenda.
 */
public interface LegendListener {
	/**
	 * Invocado cuando se cambia en el legend el valor de el campo de
	 * etiquetado. Si cualquiera de los listeners devuelve false, el cambio no
	 * se realiza
	 *
	 * @param e LabelLegendEvent
	 *
	 * @return True si ha sido correcto.
	 */
	boolean labelFieldChange(LabelLegendEvent e);

	/**
	 * Invocado cuando se cambia en el legend el smbolo por defecto. Si
	 * cualquiera de los listeners devuelve false, el cambio no se realiza
	 *
	 * @param e SymbolLegendEvent
	 *
	 * @return True si ha sido correcto.
	 */
	boolean defaultSymbolChange(SymbolLegendEvent e);

	/**
	 * Invocado cuando se cambia en el legend uno de los smbolos. El legend
	 * ser de tipo "valores nicos" o de tipo "intervalos". Si cualquiera de
	 * los listeners devuelve false, el cambio no se realiza
	 *
	 * @param e SymbolLegendEvent
	 *
	 * @return True si ha sido correcto.
	 */
	boolean classifiedSymbolChange(SymbolLegendEvent e);

	/**
	 * Invocado cuando se cambia en un legend por intervalos el valor de uno de
	 * los intervalos. Si cualquiera de los listeners devuelve false, el
	 * cambio no se realiza
	 *
	 * @param e IntervalLegendEvent
	 *
	 * @return True si ha sido correcto.
	 */
	boolean intervalChange(IntervalLegendEvent e);

	/**
	 * Invocado cuando se cambia en un legend de valores nicos, el valor de
	 * uno de los valores. Si cualquiera de los listeners devuelve false, el
	 * cambio no se realiza
	 *
	 * @param e ValueLegendEvent
	 *
	 * @return True si ha sido correcto.
	 */
	boolean valueChange(ValueLegendEvent e);
}
