/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.RectangleEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.RectangleListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;


/**
 * Behaviour que espera un listener de tipo RectangleListener.
 *
 * @author Vicente Caballero Navarro
 */
public class RectangleBehavior extends Behavior {
	private Point2D m_FirstPoint;
	private Point2D m_LastPoint;
	private Point2D m_PointAnt;
	private RectangleListener listener;

	/**
	 * Crea un nuevo RectangleBehavior.
	 *
	 * @param zili listener.
	 */
	public RectangleBehavior(RectangleListener zili) {
		listener = zili;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		///g.setColor(Color.white);
		///g.fillRect(0, 0, getMapControl().getWidth(), getMapControl().getHeight());
		BufferedImage img = getMapControl().getImage();
		g.drawImage(img, 0, 0, null);
		g.setColor(Color.black);
		g.setXORMode(Color.white);

		// Borramos el anterior
		Rectangle r = new Rectangle();

		// Dibujamos el actual
		if ((m_FirstPoint != null) && (m_LastPoint != null)) {
			r.setFrameFromDiagonal(m_FirstPoint, m_LastPoint);
			g.drawRect(r.x, r.y, r.width, r.height);
		}
		g.setPaintMode();
	}

	/**
	 * Reimplementacin del mtodo mousePressed de Behavior.
	 *
	 * @param e MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		Point pScreen = e.getPoint();
		m_PointAnt = pScreen;
		m_FirstPoint = m_PointAnt;

		if (e.getButton() == MouseEvent.BUTTON1) {
			m_PointAnt = pScreen;
			m_FirstPoint = m_PointAnt;
		}

		if (listener.cancelDrawing()) {
			getMapControl().cancelDrawing();
		}

		getMapControl().repaint();
	}

	/**
	 * Reimplementacin del mtodo mouseReleased de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
	    if (m_FirstPoint == null) return;
		Point2D p1;
		Point2D p2;
		Point pScreen = e.getPoint(); 

		ViewPort vp = getMapControl().getMapContext().getViewPort();

		p1 = vp.toMapPoint(m_FirstPoint);
		p2 = vp.toMapPoint(pScreen);

		if (e.getButton() == MouseEvent.BUTTON1) {
			//	Fijamos el nuevo extent
			Rectangle2D.Double r = new Rectangle2D.Double();
			r.setFrameFromDiagonal(p1, p2);

			Rectangle2D rectPixel = new Rectangle();
			rectPixel.setFrameFromDiagonal(m_FirstPoint, pScreen);

			RectangleEvent event = new RectangleEvent(r, e, rectPixel);
			listener.rectangle(event);
		}

		m_FirstPoint = null;
		m_LastPoint = null;
	}

	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 */
	public void mouseDragged(MouseEvent e) {
		m_LastPoint = e.getPoint();
		getMapControl().repaint();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(com.iver.cit.gvsig.fmap.tools.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		this.listener = (RectangleListener) listener;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return listener;
	}
}
