/*
 * Created on 16-oct-2004
 */
package com.hardcode.gdbms.engine.data.indexes;

import java.io.IOException;


/**
 * Interfaz a implementar por los ndices sobre las tablas. Esta interfaz se
 * utiliza al filtrar una tabla, en la que se aaden indices a la tabla
 * secuencialmente. Una vez se invoca el mtodo indexSetComplete ya no se
 * pueden meter ms ndices
 *
 * @author Fernando Gonzlez Corts
 */
public interface VariableIndexSet extends BaseIndexSet {
	/**
	 * Cierra el conjunto de ndices para el almacenamiento, a partir de una
	 * llamada a este mtodo no se puede escribir ningn ndice ms, slamente
	 * se pueden leer
	 *
	 * @throws IOException Si se produce un fallo al cerrar los streams de
	 * 		   salida
	 */
	public void indexSetComplete() throws IOException;

	/**
	 * Aade un ndice al conjunto de ndices
	 *
	 * @param value ndice de la fila a la que apunta el ndice que se quiere
	 * 		  aadir
	 *
	 * @throws IOException Si se produce un fallo al escribir el ndice
	 */
	public void addIndex(long value) throws IOException;

	/**
	 * Abre el almacenamiento del ndice para la escritura de los ndices. En
	 * caso de un almacenamiento permanente se usar un fichero temporal
	 *
	 * @throws IOException Si se produce un fallo al abrir
	 */
	public void open() throws IOException;

	/**
	 * Obtiene los ndices del conjunto de ndices en un array
	 *
	 * @return long[]
	 *
	 * @throws IOException
	 */
	public long[] getIndexes() throws IOException;
}
