package com.hardcode.gdbms.engine.instruction;

import com.hardcode.gdbms.engine.values.Value;


/**
 * Interfaz a implementar sobre los nodos
 *
 * @author Fernando Gonzlez Corts
 */
public interface Expression {
    /**
     * Evala la expresin para la fila row y realiza el cacheado del resultado
     * en caso de que la expression sea un literal
     *
     * @param row fila para la que se evalua la expresin
     *
     * @return Valor resultante de evaluar la expresin para la fila row
     *
     * @throws EvaluationException Si se produce algn error semntico
     */
    Value evaluateExpression(long row) throws EvaluationException;

    /**
     * Obtiene el nombre del campo en el que consiste la expresin. En el caso
     * de que la expresin conste de alguna operacin o no contenga ninguna
     * referencia a un campo se devolver null.
     *
     * @return Nombre del campo
     */
    String getFieldName();

    /**
     * Checks if this expression is an aggregate function. It is, implements
     * the Function interface and its isAggregate method returns true
     *
     * @return boolean
     */
    boolean isAggregated();

    /**
     * Simplifica las expresiones del rbol de adaptadores
     */
    void simplify();

    /**
     * Evala la expresin para la fila row
     *
     * @param row fila para la que se evalua la expresin
     *
     * @return Valor resultante de evaluar la expresin para la fila row
     *
     * @throws EvaluationException Si se produce algn error semntico
     */
    Value evaluate(long row) throws EvaluationException;

    /**
     * Indica si los operandos de esta expresin son siempre los mismos o
     * pueden cambiar. Puede cambiar cuando el operando es una funcion o una
     * referencia a un campo y no debe cambiar en el resto de casos
     *
     * @return true si esta expresin va a devolver siempre el mismo valor
     */
    boolean isLiteral();
}
