/*
 * Created on 30-ago-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.hardcode.gdbms.engine.spatial.fmap;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.SpatialFileDriver;
import com.hardcode.gdbms.engine.data.file.FileDataWare;
import com.hardcode.gdbms.engine.spatial.Geometry;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.VectorialFileAdapter;


public abstract class AbstractFMapFileDriver implements SpatialFileDriver {

	private VectorialAdapter adapter;
	private DataSource dataSource;

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.FileDriver#open(java.io.File)
	 */
	public void open(VectorialFileAdapter adapter) throws IOException {
		try {
			this.adapter = adapter;
			adapter.start();
			dataSource = adapter.getRecordset();
			dataSource.start();
		} catch (DriverIOException e) {
			throw new IOException(e.getMessage());
		} catch (DriverException e) {
			throw new IOException(e.getMessage());
		} catch (DriverLoadException e) {
			throw new IOException(e.getMessage());
		}
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.FileDriver#close()
	 */
	public void close() throws IOException {
		try{
			adapter.stop();
			dataSource.stop();
		} catch (DriverIOException e) {
			throw new IOException(e.getMessage());
		} catch (DriverException e) {
			throw new IOException(e.getMessage());
		}
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.FileDriver#writeFile(com.hardcode.gdbms.engine.data.file.FileDataWare, java.io.File)
	 */
	public void writeFile(FileDataWare dataWare) throws DriverException {
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.FileDriver#createSource(java.lang.String, java.lang.String[], int[])
	 */
	public void createSource(String path, String[] fieldNames, int[] fieldTypes) throws IOException {
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldValue(long, int)
	 */
	public Value getFieldValue(long rowIndex, int fieldId) throws DriverException {
		return dataSource.getFieldValue(rowIndex, fieldId);
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldCount()
	 */
	public int getFieldCount() throws DriverException {
		return dataSource.getFieldCount();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldName(int)
	 */
	public String getFieldName(int fieldId) throws DriverException {
		return dataSource.getFieldName(fieldId);
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getRowCount()
	 */
	public long getRowCount() throws DriverException {
		return dataSource.getRowCount();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldType(int)
	 */
	public int getFieldType(int i) throws DriverException {
		return dataSource.getFieldType(i);
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.SpatialDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws DriverException {
		try {
			return adapter.getFullExtent();
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.SpatialDriver#getGeometry(long)
	 */
	public Geometry getGeometry(long rowIndex) throws DriverException {
		try {
			return new FMapGeometry(adapter.getShape((int)rowIndex));
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.SpatialDriver#getJTSGeometry(long)
	 */
	public com.vividsolutions.jts.geom.Geometry getJTSGeometry(long rowIndex) throws DriverException {
		try {
			IGeometry ig = adapter.getShape((int)rowIndex);
			GeneralPathX gpx = new GeneralPathX();
			gpx.append(ig.getPathIterator(null), true);
			return FConverter.java2d_to_jts(new FShapeGeneralPathX(gpx, ig.getGeometryType()));
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}
	
	protected VectorialAdapter getAdapter() {
		return adapter;
	}
	
	protected DataSource getDataSource() {
		return dataSource;
	}
	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldWidth(int)
	 */
	public int getFieldWidth(int i) throws DriverException {
		return dataSource.getFieldWidth(i);
	}

}
