package org.gvsig.remoteClient.gml.factories;

import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;

import org.gvsig.remoteClient.gml.GMLFeaturesParser;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.gml.v2.GMLFeaturesParser_v2;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FeaturesParserFactory.java,v 1.1 2006/08/10 12:00:49 jorpiell Exp $
 * $Log: FeaturesParserFactory.java,v $
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * This class has all the classes to parse different types 
 * of GML files. 
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FeaturesParserFactory {
	private static Hashtable supportedVersions = new Hashtable();
	
	static {
		supportedVersions.put("2.0.0",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.1.1",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.1.2",GMLFeaturesParser_v2.class);
	}
	
	/**
	 * Gets the fatures iterator
	 * @param version
	 * Depending of this value, a different parser will be
	 * created and returned
	 * @param factory
	 * Factory to create the geometries
	 * @param parser
	 * Used to parse the GML file
	 * @return
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 */
	public GMLFeaturesParser createParser(String version,
			IGeometriesFactory factory,
			XMLSchemaParser parser			
			) throws Exception{
		Class clazz = (Class) supportedVersions.get(version);
		if (clazz != null){
			Class [] args = {IGeometriesFactory.class,XMLSchemaParser.class};
			Object [] params = {factory,parser};
			GMLFeaturesParser gmlParser = (GMLFeaturesParser) clazz.getConstructor(args).newInstance(params);
			return gmlParser;
		}
		return null;		
	}
	
	
}
