package org.gvsig.gui.beans.controls.combobutton;



import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.GeneralPath;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;

import org.gvsig.gui.beans.controls.IControl;


public class ComboButton extends JPanel implements IControl {

	ModeToggleButtonGroup bg;
	private MyJToggleButton tbutton;
	private JPopupMenu popMenu;
	private ArrayList menuItemList;
	private ArrayList actionListeners = new ArrayList();

	private ActionListener popupMenuItemListener;
	private ArrayList buttons=new ArrayList();
	int size;

	public ComboButton() {
		//this.app = app;
		this.bg = new ModeToggleButtonGroup();

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		tbutton = new MyJToggleButton(this);
		tbutton.setAlignmentY(BOTTOM_ALIGNMENT);
		add(tbutton);

		popMenu = new JPopupMenu();
		menuItemList = new ArrayList();
		popupMenuItemListener = new MenuItemListener();
		size = 0;
	}

	public JToggleButton getJToggleButton() {
		return tbutton;
	}

	public boolean selectMode(int mode) {
		String modeText = mode + "";

		for (int i=0; i < size; i++) {
			JMenuItem mi = (JMenuItem) menuItemList.get(i);
			// found item for mode?
			if (mi.getActionCommand().equals(modeText)) {
				selectItem(mi);
				return true;
			}
		}
		return false;
	}

	private void selectItem(JMenuItem mi) {
		tbutton.setIcon(mi.getIcon());
		tbutton.setToolTipText(mi.getText());
		tbutton.setActionCommand(mi.getActionCommand());
		tbutton.setSelected(true);
		tbutton.requestFocus();
	}

	public void addButton(JButton menu) {
		buttons.add(menu);
		JMenuItem mi = new JMenuItem();
		mi.setText(menu.getText());
		mi.setIcon(menu.getIcon());
		mi.addActionListener(popupMenuItemListener);
		mi.setActionCommand(menu.getActionCommand());
		popMenu.add(mi);
		menuItemList.add(mi);
		size++;

		if (size == 1) {
			// init tbutton
			tbutton.setIcon(menu.getIcon());
			tbutton.setActionCommand(menu.getActionCommand());
			tbutton.setToolTipText(mi.getText());
			// add button to button group
			bg.add(tbutton);
		}

		// set the action listeners
		Iterator actionListenerIterator = actionListeners.iterator();
		while (actionListenerIterator.hasNext()) {
			ActionListener listener = (ActionListener) actionListenerIterator.next();
			menu.addActionListener(listener);
		}
	}

	public void addSeparator() {
		popMenu.addSeparator();
	}

	// sets new mode when item in popup menu is selected
	private class MenuItemListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			JMenuItem item = (JMenuItem) e.getSource();
			selectItem(item);
			tbutton.doClick();
		}
	}

	public void mouseOver() {
		//	popup menu is showing
		JPopupMenu activeMenu = bg.getActivePopupMenu();
		if (activeMenu != null && activeMenu.isShowing()) {
			setPopupVisible(true);
		}
	}

	//	shows popup menu
	public void setPopupVisible(boolean flag) {
		if (flag) {
			bg.setActivePopupMenu(popMenu);
			if (popMenu.isShowing()) return;
			Point locButton = tbutton.getLocationOnScreen();
			Point locApp = this.getLocationOnScreen();
			popMenu.show(this, locButton.x - locApp.x,
											locButton.y - locApp.y + tbutton.getHeight());
		} else {
			popMenu.setVisible(false);
		}
	}

	public boolean isPopupShowing() {
		return popMenu.isShowing();
	}

	public void setAction(ActionEvent action) {
		JButton[] bs=(JButton[])buttons.toArray(new JButton[0]);
		for (int i=0;i<bs.length;i++) {
			if (bs[i].getActionCommand().equals(action.getActionCommand())){
				bs[i].doClick();
			}
		}

	}

	/**
	 * Adds an ActionListener to each button in the combo.
	 *
	 * @param listener
	 */
	public void addActionListener(ActionListener listener) {
		if (!actionListeners.contains(listener)) {
			actionListeners.add(listener);
			Iterator buttonsIterator = buttons.iterator();
			while (buttonsIterator.hasNext()) {
				JButton button = (JButton) buttonsIterator.next();
				button.addActionListener(listener);
			}
		}
	}

	/**
	 * Removes an ActionListener from each button in the combo.
	 * @param listener
	 */
	public void removeActionListener(ActionListener listener) {
		actionListeners.remove(listener);
		Iterator buttonsIterator = buttons.iterator();
		while (buttonsIterator.hasNext()) {
			JButton button = (JButton) buttonsIterator.next();
			button.removeActionListener(listener);
		}
	}

	/**
	 *  Sets the value of the control. It may mean different things for
	 *  different kinds of controls.
	 */
	public Object setValue(Object value) {
		//TODO implement this method
		return value;
	}
}

class MyJToggleButton extends JToggleButton
implements MouseListener, MouseMotionListener, ActionListener {
	private static int BORDER = 6;
	private int iconWidth, iconHeight;
	private GeneralPath gp;
	private boolean showToolTipText = true;
	private ComboButton menu;
	private Color arrowColor = new Color(0,0,0,130);
	//private Color arrowColor = Color.darkGray;

	MyJToggleButton(ComboButton menu) {
		super();
		this.menu = menu;

		// add own listeners
		addMouseListener(this);
		addMouseMotionListener(this);
		addActionListener(this);
	}

	public String getToolTipText() {
		if (showToolTipText)
			return super.getToolTipText();
		else
			return null;
	}

	// set mode
	public void actionPerformed(ActionEvent e) {
		menu.setAction(e);
	}

	public void setIcon(Icon icon) {
		super.setIcon(icon);
		iconWidth = icon.getIconWidth();
		iconHeight = icon.getIconHeight();
		Dimension dim = new Dimension(iconWidth + 2*BORDER,
								iconHeight + 2*BORDER);
		setPreferredSize(dim);
		setMinimumSize(dim);
		setMaximumSize(dim);
	}


	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;

		super.paint(g2);

		// draw little arrow (for popup menu)
		if (menu.size > 1) {
			if (gp == null) initPath();
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								RenderingHints.VALUE_ANTIALIAS_ON);

			g2.setColor(Color.white);
			g2.fill(gp);
			g2.setColor(arrowColor);
			g2.draw(gp);
		}
	}

	private void initPath() {
		gp = new GeneralPath();
		int x = BORDER + iconWidth + 2;
		int y = BORDER + iconHeight + 1;
		gp.moveTo(x-6, y-5);
		gp.lineTo(x, y-5);
		gp.lineTo(x-3,y);
		gp.closePath();
	}


	private boolean popupTriangleClicked(int x, int y) {
		return (menu.size > 1 && y > iconHeight);
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

	public void mousePressed(MouseEvent e) {
		boolean showPopup=popupTriangleClicked(e.getX(), e.getY());
		menu.setPopupVisible(showPopup);
		requestFocus();
		if (!showPopup)
			doClick();
	}

	public void mouseReleased(MouseEvent arg0) {
	}

	public void mouseDragged(MouseEvent e) {
		if (popupTriangleClicked(e.getX(), e.getY()))
			menu.setPopupVisible(true);
	}

	public void mouseMoved(MouseEvent e) {
		menu.mouseOver();
		showToolTipText = !menu.isPopupShowing();
	}
}

