
package org.agil.core.spatialindex;

import java.io.IOException;
import java.util.List;

import com.vividsolutions.jts.geom.Envelope;
/**
 * Ttulo: FileSpatialIndex
 * Descripcion:
 * <p>
 * Es un indice espacial en fichero.<br>
 * A diferencia del indice espacial normal, que puede trabajar
 * en memoria, se debe ocupar de ocupar y/o liberar los recursos
 * asociados.<br>
 * Esto se concreta en que se han aadido los metodos create() y
 * close(), para respectivamente crear un fichero de indice, y para
 * cerrar el canal de I/O asociado.
 * </p>
 *
 * Observaciones:
 * <p>
 * Un buen diseo hara que heredase de com.vividsolutions.jump.index.SpatialIndex.<br>
 * El problema es que, por la naturaleza de este indice, est asociado a ficheros
 * y debe poder lanzar excepciones I/O. Como SpatialIndex de JTS no las lanza,
 * de momento mantenemos 2 interfaces
 *
 * @tododo Unificar la interfaz SpatialIndex y FileSpatialIndex
 * </p>
 *
 * Copyright:    Copyright (c) 2003
 * @version 1.0
 */

public interface FileSpatialIndex  {

    public void open() throws IOException;
    /**
     * Metodo que se encarga de crear, solo la primera vez, el fichero de indice espacial.
     * Lo normal es que esto se haga desde scripts bath (en un proceso de carga de indices)
     * o por peticion.
     * */
    public void createSpatialIndex() throws IOException;
    /**
     * Libera los recursos consumidos -canales de I/O- por el fichero
     * de indice
     * */
    public void close() throws IOException;

    /**
   * Adds a spatial item to the index with the given envelope
   */
    public void insert(Envelope itemEnv, Object item) throws IOException;

    /**
     * Queries the index for all items whose envelopes intersect the given search envelope
     *
     * @param searchEnv the envelope to query for
     * @return a list of the items found by the query
     */
    public List query(Envelope searchEnv) throws IOException;




}